<?php

namespace Helpie\Features\Components\Category_Listing;

use \Helpie\Includes\Translations as Translations;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Helpie\Features\Components\Category_Listing\Fields_Model')) {
    class Fields_Model
    {
        public function __construct()
        {
            $this->category_repo = new \Helpie\Features\Domain\Repositories\Category_Repository();
        }

        public function get_fields()
        {
            $fields = array(
                'title' => $this->get_title_field(),
                'show_widget_title' => $this->get_show_widget_title(),
                'sortby' => $this->get_sortby_field(),
                'topics' => $this->get_topics_field(),
                'num_of_cols' => $this->get_num_of_cols_field(),
                'type' => $this->get_type_field(),
                'children_type' => $this->get_children_type_field(),
                'num_of_articles' => $this->get_num_of_articles_field(),
                'show_image' => $this->get_show_image_field(),
                'show_description' => $this->get_show_description_field(),
            );

            return $fields;
        }

        public function get_default_args()
        {
            $args = array();

            // Get Default Values from GET - FIELDS
            $fields = $this->get_fields();
            foreach ($fields as $key => $field) {
                $args[$key] = $field['default'];
            }

            return $args;
        }

        // FIELDS
        protected function get_title_field()
        {
            return array(
                'name' => 'title',
                'label' => Translations::get('title'),
                'default' => 'KB Category Listing',
                'type' => 'text',
            );
        }

        protected function get_topics_field()
        {
            $categories_options = $this->category_repo->get_category_options(true); // $show_all = true

            return array(
                'name' => 'topics',
                'label' => Translations::get('topics'),
                'default' => 'all',
                'options' => $categories_options,
                'type' => 'multi-select',
            );
        }

        protected function get_show_widget_title()
        {
            return array(
                'name' => 'show_widget_title',
                'label' => Translations::get('display_widget_title'),
                'default' => 'false',
                'options' => array(
                    'true' => Translations::get('true'),
                    'false' => Translations::get('false'),
                ),
                'type' => 'select',
            );
        }

        protected function get_sortby_field()
        {
            return array(
                'name' => 'sortby',
                'label' => Translations::get('sort_by'),
                'default' => 'custom',
                'options' => array(
                    'alphabetical' => Translations::get('alphabetical'),
                    'count' => Translations::get('article_count'),
                    'custom' => Translations::get('custom'),
                ),
                'type' => 'select',
            );
        }

        protected function get_num_of_cols_field()
        {
            return array(
                'name' => 'num_of_cols',
                'label' => Translations::get('num_of_cols'),
                'default' => 'three',
                'options' => array(
                    'one' => 1,
                    'two' => 2,
                    'three' => 3,
                    'four' => 4,
                ),
                'type' => 'select',
            );
        }

        protected function get_type_field()
        {
            return array(
                'name' => 'type',
                'label' => Translations::get('type'),
                'default' => 'boxed',
                'options' => array(
                    'boxed' => Translations::get('boxed'),
                    'boxed1' => Translations::get('boxed1'),
                    'modern' => Translations::get('modern'),
                    'list' => Translations::get('list'),
                ),
                'type' => 'select',
            );
        }

        protected function get_children_type_field()
        {
            return array(
                'name' => 'children_type',
                'label' => Translations::get('children_type'),
                'default' => 'articles',
                'options' => array(
                    'none' => Translations::get('dont_show_children'),
                    'articles' => Translations::get('articles'),
                    'sub-categories' => Translations::get('sub_categories'),
                ),
                'type' => 'select',
            );
        }

        protected function get_num_of_articles_field()
        {
            return array(
                'name' => 'num_of_articles',
                'label' => Translations::get('num_of_articles'),
                'default' => 5,
                'type' => 'number',
            );
        }

        protected function get_show_description_field()
        {
            return array(
                'name' => 'show_description',
                'label' => Translations::get('show_description'),
                'default' => 'true',
                'options' => array(
                    'true' => Translations::get('true'),
                    'false' => Translations::get('false'),
                ),
                'type' => 'select',
            );
        }

        protected function get_show_image_field()
        {
            return array(
                'name' => 'show_image',
                'label' => Translations::get('show_image'),
                'default' => 'true',
                'options' => array(
                    'true' => Translations::get('true'),
                    'false' => Translations::get('false'),
                ),
                'type' => 'select',
            );
        }
    }
}
