<?php

namespace Helpie\Features\Components\Toc\Model;

use \Helpie\Includes\Translations as Translations;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Helpie\Features\Components\Toc\Model\Fields_Model')) {
    class Fields_Model
    {
        public function __construct()
        {
            $this->category_repo = new \Helpie\Features\Domain\Repositories\Category_Repository();
        }

        public function get_fields()
        {
            $fields = array(
                'title' => $this->get_title_field(),
                'toc_type' => $this->get_toc_type_field(),
                'show_auto_toc' => $this->get_show_auto_toc_field(),
                'show_toc_articles' => $this->get_show_toc_articles_field(),
                'toggle_category_children' => $this->get_toggle_category_children_field(),
                'article_limit' => $this->get_article_limit_field(),
                'topics' => $this->get_topics_field(),
            );

            return $fields;
        }

        // FIELDS

        protected function get_article_limit_field()
        {
            return array(
                'name' => 'article_limit',
                'label' => Translations::get('article_limit'),
                'default' => 5,
                'type' => 'number',
            );
        }

        protected function get_upto_child_category_limit_field()
        {
            return array(
                'name' => 'child_category_limit',
                'label' => Translations::get('child_category_limit'),
                'default' => 5,
                'type' => 'number',
            );
        }

        protected function get_toggle_category_children_field()
        {
            return array(
                'name' => 'toggle_category_children',
                'label' => Translations::get('toggle_category_children'),
                'default' => 'true',
                'options' => array(
                    'true' => Translations::get('true'),
                    'false' => Translations::get('false'),
                ),
                'type' => 'select',
            );
        }

        protected function get_show_auto_toc_field()
        {
            return array(
                'name' => 'show_auto_toc',
                'label' => Translations::get('show_auto_toc'),
                'default' => 'true',
                'options' => array(
                    'true' => Translations::get('true'),
                    'false' => Translations::get('false'),
                ),
                'type' => 'select',
            );
        }

        protected function get_show_toc_articles_field()
        {
            return array(
                'name' => 'show_toc_articles',
                'label' => Translations::get('toc_display_articles'),
                'default' => 'true',
                'options' => array(
                    'true' => Translations::get('true'),
                    'false' => Translations::get('false'),
                ),
                'type' => 'select',
            );
        }

        protected function get_title_field()
        {
            return array(
                'name' => 'title',
                'label' => Translations::get('title'),
                'default' => 'Table of Contents',
                'type' => 'text',
            );
        }

        protected function get_toc_type_field()
        {
            return array(
                'name' => 'toc_type', // This is different in settings
                'label' => Translations::get('type'),
                'default' => 'full-nav',
                'options' => array(
                    'full-nav' => Translations::get('toc_full_nav'),
                    'page-scroll-only' => Translations::get('toc_scroll_only'),
                ),
                'type' => 'select',
            );
        }

        protected function get_topics_field()
        {
            $categories_options = $this->category_repo->get_category_options(true); // $show_all = true

            return array(
                'name' => 'topics',
                'label' => Translations::get('topics'),
                'default' => 'all',
                'options' => $categories_options,
                'type' => 'multi-select',
            );
        }
    } // END CLASS
}
