<?php

namespace Helpie\Features\Services\Access_Control\Strategies;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Helpie\Features\Services\Access_Control\Strategies\Filter_Get_Terms')) {
    class Filter_Get_Terms
    {

        public function __construct($content)
        {
            $this->access_helper = new \Helpie\Features\Services\Access_Control\Helper();
            // $this->query = $query;
            $this->content = $content;
            $this->settings = new \Helpie\Includes\Settings\Getters\Settings();
            $this->terms = null;
        }

        public function filter($terms, $taxonomies, $args, $term_query)
        {
            // error_log('Filter_Get_Terms->filter');
            // error_log('$terms : ' . print_r($terms, true));

        //     $are_valid_terms = true;
        //     foreach ($terms as $key => $term) {
        //         if(!isset($term) || !isset($term->term_id)){
        //             $are_valid_terms = false;
        //             break;
        //         }
        //     }

        //    if(!$are_valid_terms){
        //         return $terms;
        //    }

            /* Filtering criteria */
            $terms = $this->dynamic_caps_based_filter($terms);
            $terms = $this->dnd_exclude($terms);

            return $terms;
            // error_log('end Filter_Get_Terms $this->query : ' . print_r($this->query, true));
        }

        protected function dnd_exclude($terms)
        {
            $helpie_mp_cats = $this->settings->main_page->get_mp_cats();
            $disabled = $helpie_mp_cats['disabled'];
            $disabled_term_ids = [];

            foreach ((array) $disabled as $key => $value) {
                $disabled_term_ids[] = str_replace('term-id_', '', $key);
            }

            foreach ($terms as $ii => $term) {

                if(in_array($term->term_id, $disabled_term_ids)){
                    unset($terms[$ii]);
                }
               
            }

            // error_log('$disabled_term_ids : ' . print_r($disabled_term_ids, true));
            return $terms;

        }


        public function dynamic_caps_based_filter($terms){
            // error_log('$terms : ' . print_r($terms, true));
            // error_log('$this->content : ' . print_r($this->content, true));

            $is_included_empty = $this->is_included_empty();
            $is_excluded_empty = $this->is_excluded_empty();

            // 1. Global = 1, included & excluded are empty
            if($this->content['global'] == 1 && $is_included_empty && $is_excluded_empty){
                // error_log('EARLY RETURN: ALL EMPTY');
                return $terms;
            }

            // $filtered_terms = [];
            // error_log('$terms : ' . print_r($terms, true));
            foreach ($terms as $ii => $term) {
                $is_valid_term = (isset($term) && isset($term->term_id));

                // error_log('$term : ' . print_r($term, true));
                // error_log('($term instanceof WP_Term): ' . ($term instanceof WP_Term));
                // error_log('$is_valid_term: ' . $is_valid_term);

                if (!$is_valid_term) {
                    unset($terms[$ii]);
                    continue;
                }

                // $can_remove_term = true;
                $can_remove_term = $this->can_remove_term($term);

                if($can_remove_term){
                    unset($terms[$ii]);
                }

            }

            $terms = array_values($terms); // 'reindex'
            
            // error_log('$terms : ' . print_r($terms, true));
            return $terms;
        }

        protected function is_included_empty(){
            $is_included_empty = false;

            if( empty($this->content['included']['topics']) &&  empty($this->content['included']['posts']) ){
                $is_included_empty = true;
            }

            return $is_included_empty;
        }

        protected function is_excluded_empty(){
            $is_excluded_empty = false;

            if( empty($this->content['excluded']['topics']) &&  empty($this->content['excluded']['posts']) ){
                $is_excluded_empty = true;
            }

            return $is_excluded_empty;
        }


        protected function can_remove_term($term){
            // $this->is_topic_descendant_included($term);
            // echo "term: ";
            // print_r($term);

            $topic_included = $this->is_topic_included($term);

            if($topic_included){
                return false;
            }

            $topic_excluded = $this->is_topic_excluded($term);
            
            /* Dummy */
            // $topic_descendant_included = false;
            // $post_included = false;
            /* Dummy END */
            
            $topic_descendant_included = $this->is_topic_descendant_included($term);  // child_topics and their posts
            $post_included = $this->has_included_posts($term->term_id);  // posts of this topic alone
            $topic_neutral = $this->is_topic_neutral($term);
            $ancestor_excluded = $this->access_helper->is_term_excluded($term->term_id, $this->content);
            $ancestor_included = $this->access_helper->is_term_included($term->term_id, $this->content);
            $global_allowed = $this->is_global_allowed();


            $topic_excluded_condition = ($topic_excluded && (!$topic_descendant_included && !$post_included) );
            $topic_neutral_condition1 = !$ancestor_included && (!$global_allowed || ($global_allowed && $ancestor_excluded) );
            $topic_neutral_condition2 = (!$topic_descendant_included && !$post_included);
            $topic_neutral_condition = $topic_neutral && $topic_neutral_condition1 && $topic_neutral_condition2;
            
            // Final Condition
            $remove_condition = ($topic_excluded_condition) || ($topic_neutral_condition);
            
            if($term->term_id == 33){
                // error_log('can_remove_term term_id: ' .  $term->term_id);
                // error_log('remove_condition: ' . $remove_condition);
            
                // error_log('topic_excluded_condition: ' . $topic_excluded_condition);
                // error_log('topic_neutral_condition: ' . $topic_neutral_condition);
                // error_log('topic_neutral_condition1: ' . $topic_neutral_condition1);
                // error_log('topic_neutral_condition2: ' . $topic_neutral_condition2);
                // error_log('ancestor_included: ' . $ancestor_included);
                // error_log('global_allowed: ' . $global_allowed);
                // error_log('$topic_descendant_included: ' . $topic_descendant_included);
            }


            return $remove_condition;
        }
        
        protected function has_included_posts($term_id)
        {
            $has_included_posts = false;
            $included_posts = (array) $this->content['included']['posts'];

            // $query_service = \Helpie\Features\Services\Query::getInstance();
            // return $query_service->has_included_posts($term_id, $included_posts);

            global $helpie_db;
            $terms = $helpie_db->get_all_terms();

            // error_log('helpie_db $terms : ' . print_r($terms, true));
            // error_log('term_id: ' . $term_id);

            if(isset($terms[$term_id])){
                $posts = $terms[$term_id];
            }
            

            if (!isset($posts) || !is_array($posts)) {
                return $has_included_posts;
            }

            foreach ($posts as $post) {
                if (in_array($post['ID'], $included_posts)) {
                    $has_included_posts = true;
                    break;
                }
            }

            return $has_included_posts;

        }


        protected function is_topic_included($term){
            return in_array($term->term_id, $this->content['included']['topics']);
        }

        protected function is_topic_excluded($term){
            return in_array($term->term_id, $this->content['excluded']['topics']);
        }

        protected function is_topic_neutral($term){
            $is_topic_included = $this->is_topic_included($term);
            $is_topic_excluded = $this->is_topic_excluded($term);

            return (!$is_topic_included && !$is_topic_excluded);
        }

        protected function is_global_allowed(){
            return ($this->content['global'] == 1);
        }

        protected function is_topic_descendant_included($term){
            $result = false;
            $hierarchy = _get_term_hierarchy('helpdesk_category');

            $result = $this->is_any_topic_descendant_included_recursive($term->term_id, $hierarchy);
            
            return $result;
            // error_log('$hierarchy : ' . print_r($hierarchy, true));
        }

        protected function is_any_topic_descendant_included_recursive($term_id, $hierarchy){
            $result = false;
            if( isset($hierarchy[$term_id]) ){
                $current_term = $hierarchy[$term_id];

                foreach ($current_term as $key => $child_term_id) {
                    $topic_result = in_array($child_term_id, $this->content['included']['topics']);
                    $post_result = $this->has_included_posts($child_term_id);

                    $result = $topic_result || $post_result;

                    if($result == true){
                        break;
                    } else{
                        $result = $this->is_any_topic_descendant_included_recursive($child_term_id, $hierarchy);
                    }
                }                
            }

            return $result;
        }



        protected function can_include_term($term){
            $topic_allowed = in_array($term->term_id, $this->content['included']['topics']);
            $child_term_allowed = $this->is_any_child_term_allowed($term);
            $post_allowed = false;

            $global_false_condition = $global_disallowed && 
            (   ($topic_allowed) || 
                ((!$topic_allowed) && ($child_term_allowed || $post_allowed))
            );


            $show_term = $global_false_condition;

            // error_log('term_id: ' . $term->term_id);
            // error_log('global_disallowed: ' . $global_disallowed);
            // error_log('topic_allowed: ' . $topic_allowed);
            // error_log('child_term_allowed: ' . $child_term_allowed);
            // error_log('$show_term: ' . $show_term);

            if($show_term){
                $filtered_terms[] = $term;
            }

            return $this->is_term_excluded($term->parent);
        }
        

        protected function is_any_child_term_allowed($term){
            $child_terms = get_terms( array(
                'taxonomy' => HELPIE_TAXONOMY,
                'parent' => $term->term_id,
                'hide_empty' => false,
                'suppress_filter' => true,
            ) );

            $child_term_allowed = false;
            foreach ($child_terms as $key => $term) {
                $child_term_allowed = in_array($term->term_id, $this->content['included']['topics']);

                if($child_term_allowed){
                    break;
                }
            }

            // error_log('child_terms : ' . print_r($child_terms, true));

            return $child_term_allowed;
        }

    } // END CLASS
}