<?php

/**
 * Implements WPDB Queries directly
 *
 * @package    helpie-kb
 * @subpackage services
 * @author     essekia
 * @version    1.9.1
 * ...
 */

namespace Helpie\Features\Services;

if (!class_exists('\Helpie\Features\Services\DB')) {
    class DB
    {

        // Hold the class instance.
        private static $instance = null;
        private static $posts = null;
        private static $terms = null;
        private static $post_meta = null;
        private static $term_meta = null;

        // The constructor is private
        // to prevent initiation with outer code.
        private function __construct()
        {
            // The expensive process (e.g.,db connection) goes here.
        }

        // The object is created from within the class itself
        // only if the class has no instance.
        public static function getInstance()
        {
            if (self::$instance == null) {
                self::$instance = new \Helpie\Features\Services\DB();
            }

            return self::$instance;
        }

        public static function get_all_terms()
        {

            // return self::$terms;
            if (isset(self::$terms) && is_array(self::$terms)) {
                return self::$terms;
            }

            //  // Source: https://gist.github.com/minio/6982551
            global $wpdb;
            $terms_posts_result = $wpdb->get_results("
                SELECT A.term_id, A.taxonomy, B.name AS category, C.object_id, D.ID AS ID, D.post_name AS slug
                FROM $wpdb->term_taxonomy AS A
                JOIN $wpdb->terms AS B JOIN $wpdb->term_relationships AS C JOIN $wpdb->posts AS D
                WHERE A.taxonomy='" . HELPIE_TAXONOMY . "'
                    AND A.term_id = B.term_id
                    AND A.count > 0
                    AND A.term_taxonomy_id = C.term_taxonomy_id
                    AND C.object_id = D.ID
                ORDER BY category", ARRAY_A);

            foreach ($terms_posts_result as $key => $post) {
                self::$terms[$post['term_id']][] = $post;
            }

            // $terms = [];

            // self::$terms = $terms;

            // error_log('self::$terms : ' . print_r(self::$terms, true));

            return self::$terms;
        }

        public static function get_all_posts()
        {

            if (isset(self::$posts) && is_array(self::$posts)) {
                return self::$posts;
            }

            // Source: https://gist.github.com/minio/6982551
            global $wpdb;
            $posts = $wpdb->get_results("
                SELECT A.ID, D.term_id
                from $wpdb->posts AS A
                    join $wpdb->term_relationships As B on A.ID = B.object_id
                    join $wpdb->terms AS C on B.term_taxonomy_id = C.term_id
                    join $wpdb->term_taxonomy AS D on B.term_taxonomy_id = D.term_taxonomy_id
                WHERE A.post_status = 'publish' && A.post_type = '" . HELPIE_POST_TYPE . "'
                ORDER BY C.slug,A.ID
                ", ARRAY_A);

            // foreach ($posts as $key => $post) {
            //     self::$terms[$post['term_id']][] = $post;
            // }

            self::$posts = $posts;

            // error_log('$posts : ' . print_r($posts, true));
            return self::$posts;
        }

        public static function get_post_meta()
        {

            if (isset(self::$post_meta) && is_array(self::$post_meta)) {
                return self::$post_meta;
            }

            global $wpdb;
            $meta_key = '_helpie_kb_post_options';
            $all_metas = $wpdb->get_results($wpdb->prepare(
                "
                    SELECT meta_value, post_id
                    FROM $wpdb->postmeta
                    WHERE meta_key = %s
                ",
                $meta_key
            ));

            self::$post_meta = [];
            foreach ($all_metas as $key => $value) {
                self::$post_meta[$value->post_id] = unserialize($value->meta_value);
            }
            // error_log('self::$post_meta : ' . print_r(self::$post_meta, true));
            return self::$post_meta;
        }

        public function get_term_meta()
        {

            if (isset(self::$term_meta) && is_array(self::$term_meta)) {
                return self::$term_meta;
            }

            global $wpdb;
            $meta_key = '_helpie_kb_options';
            $all_metas = $wpdb->get_results($wpdb->prepare(
                "
                    SELECT meta_value, term_id
                    FROM $wpdb->termmeta
                    WHERE meta_key = %s
                ",
                $meta_key
            ));

            self::$term_meta = [];
            foreach ($all_metas as $key => $value) {
                self::$term_meta[$value->term_id] = unserialize($value->meta_value);
            }
            // error_log('self::$term_meta  : ' . print_r(self::$term_meta , true));
            return self::$term_meta;
        }
    }
}
