<?php

namespace Helpie\Features\Components\Articles;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Helpie\Features\Components\Articles\Fields_Model')) {
    class Fields_Model
    {
        public function __construct()
        {
            $this->category_repository = new \Helpie\Features\Domain\Repositories\Category_Repository();
        }

        public function get_fields()
        {
            $fields = array(
                'title' => $this->get_title_field(),
                'sortby' => $this->get_sortby_field(),
                'topics' => $this->get_topics_field(),
                'num_of_cols' => $this->get_num_of_cols_field(),
                'style' => $this->get_style_field(),
                'limit' => $this->get_limit_field(),
                'show_image' => $this->get_show_image_field(),
                'show_extra' => $this->get_show_extra_field(),
            );

            return $fields;
        }

        public function get_default_args()
        {
            $args = array();

            // Get Default Values from GET - FIELDS
            $fields = $this->get_fields();
            foreach ($fields as $key => $field) {
                $args[$key] = $field['default'];
            }

            return $args;
        }

        // FIELDS
        protected function get_title_field()
        {
            return array(
                'name' => 'title',
                'label' => __('Title', HELPIE_DOMAIN),
                'default' => __('KB Article Listing', HELPIE_DOMAIN),
                'type' => 'text',
            );
        }

        protected function get_topics_field()
        {
            $categories_options = $this->category_repository->get_category_options(true); // $show_all = true

            return array(
                'name' => 'topics',
                'label' => __('Topics', HELPIE_DOMAIN),
                'default' => __('all', HELPIE_DOMAIN),
                'options' => $categories_options,
                'type' => 'multi-select',
            );
        }

        protected function get_sortby_field()
        {
            return array(
                'name' => 'sortby',
                'label' => __('Sort By', HELPIE_DOMAIN),
                'default' => __('recent', HELPIE_DOMAIN),
                'options' => array(
                    'recent' => __('Recent', HELPIE_DOMAIN),
                    'updated' => __('Recently Updated', HELPIE_DOMAIN),
                    'popular' => __('Popular', HELPIE_DOMAIN),
                    'alphabetical' => __('Alphabetical', HELPIE_DOMAIN),
                ),
                'type' => 'select',
            );
        }

        protected function get_num_of_cols_field()
        {
            return array(
                'name' => 'num_of_cols',
                'label' => __('Num Of Columns', HELPIE_DOMAIN),
                'default' => __('three', HELPIE_DOMAIN),
                'options' => array(
                    'one' => __(1, HELPIE_DOMAIN),
                    'two' => __(2, HELPIE_DOMAIN),
                    'three' => __(3, HELPIE_DOMAIN),
                    'four' => __(4, HELPIE_DOMAIN),
                ),
                'type' => 'select',
            );
        }

        protected function get_style_field()
        {
            return array(
                'name' => 'style',
                'label' => __('Style', HELPIE_DOMAIN),
                'default' => __('list', HELPIE_DOMAIN),
                'options' => array(
                    'list' => __('List', HELPIE_DOMAIN),
                    'card' => __('Card', HELPIE_DOMAIN),
                ),
                'type' => 'select',
            );
        }

        protected function get_limit_field()
        {
            return array(
                'name' => 'limit',
                'label' => __('Limit', HELPIE_DOMAIN),
                'default' => __(5, HELPIE_DOMAIN),
                'type' => 'number',
            );
        }

        protected function get_show_image_field()
        {
            return array(
                'name' => 'show_image',
                'label' => __('Show Image', HELPIE_DOMAIN),
                'default' => __('true', HELPIE_DOMAIN),
                'options' => array(
                    'true' => __('True', HELPIE_DOMAIN),
                    'false' => __('False', HELPIE_DOMAIN),
                ),
                'type' => 'select',
            );
        }

        protected function get_show_extra_field()
        {
            return array(
                'name' => 'show_extra',
                'label' => __('Show Extra Info', HELPIE_DOMAIN),
                'default' => __('true', HELPIE_DOMAIN),
                'options' => array(
                    'true' => __('True', HELPIE_DOMAIN),
                    'false' => __('False', HELPIE_DOMAIN),
                ),
                'type' => 'select',
            );
        }

        // OTHER
    } // END CLASS
}
