<?php

namespace Helpie\Includes;

/* Dependencies */

use \Helpie\Includes\Translations as Translations;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Helpie\Includes\Settings')) {
    class Settings
    {
        public function __construct()
        {
            /* Actions */
            add_action('init', [$this, 'setup_options_init']);
            add_action('init', [$this, 'init']);

            // $this->init();

            // add_action('csf_helpie-kb_sections', [$this, 'filter_args']);

            /* Filters */
            add_filter('csf_helpie-kb_sections', [$this, 'filter_args']);
            add_filter('csf_helpie-kb_saved', [$this, 'onSettingsSave']);

            // $this->init();
            $this->extras = new \Helpie\Includes\Settings\Partials\Extras();
        }

        public function onSettingsSave($settings)
        {

            if ($settings['helpie_search_page'] == '') {
                return;
            }

            $search_page_option = 'helpdesk_search_page_id';
            update_option($search_page_option, $settings['helpie_search_page']);

            // error_log('onSettingsSave $settings : ' . print_r($settings, true));
        }

        public function filter_args($content)
        {
            return $content;
        }

        public function setup_options_init()
        {
            require_once HELPIE_PLUGIN_PATH . 'includes/settings/settings-config.php';
        }

        public function init()
        {
            $this->setup_options_init();

            if (!function_exists('\CSF') && !class_exists('\CSF')) {
                require_once HELPIE_PLUGIN_PATH . 'includes/lib/codestar-framework/codestar-framework.php';
            }

            // require_once 'settings-config.php';

            if (class_exists('\CSF')) {

                // Set a unique slug-like ID
                $prefix = 'helpie-kb';

                // Create options
                \CSF::createOptions($prefix, array(
                    'menu_title' => Translations::get('helpie_settings'),
                    'menu_parent' => 'edit.php?post_type=pauple_helpie',
                    'menu_type' => 'submenu', // menu, submenu, options, theme, etc.
                    'menu_slug' => 'helpie-kb-settings',
                    'framework_title' => Translations::get('helpie_settings'),
                    'theme' => 'light',
                    'show_search' => false, // TODO: Enable once autofill password is fixed
                ));

                // Create a section
                $this->main_page($prefix);
                $this->single_page($prefix);
                $this->category_page($prefix);
                $this->search($prefix);
                $this->frontend_editor($prefix);
                $this->styles($prefix);
                $this->password_protect($prefix);
                $this->dynamic_capabilities($prefix);
                $this->toc($prefix);
                $this->autolinking($prefix);
                $this->components($prefix);
                $this->demo_import($prefix);
                $this->backup($prefix);
                $this->performance($prefix);

                /* Additional */
                $this->can_view_posts();
                $this->can_view_topics();
                // error_log('Settings_New');
            }
        }

        public function autolinking($prefix)
        {
            // Create a section
            \CSF::createSection($prefix, array(
                'title' => Translations::get('autolinking'),
                'icon' => 'fa fa-link',
                'fields' => array(
                    array(
                        'id' => 'autolinking_enable',
                        'type' => 'switcher',
                        'title' => Translations::get('enable_autolinking'),
                        'default' => false,
                        'desc' => Translations::get('autolinking'),
                    ),
                ),
            ));
        }
        public function main_page($prefix)
        {
            $main_page_button = $this->extras->get_main_page_url();
            // Create a section
            \CSF::createSection($prefix, array(
                'title' => Translations::get('main_page'),
                'icon' => 'fa fa-home',
                'fields' => array(
                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('template'),
                    ),
                    array(
                        'id' => 'helpie_mp_sidebar_template',
                        'type' => 'image_select',
                        'title' => Translations::get('template'),
                        'options' => array(
                            'both-side-sidebars' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/both-side-sidebars.png',
                            'left-sidebar' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/left-sidebar.png',
                            'right-sidebar' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/right-sidebar.png',
                            'full-width' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/full-width.png',
                            'boxed-width' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/boxed-width.png',
                        ),
                        'default' => 'boxed-width',
                        'desc' => Translations::get('mp_template_help'),
                    ),
                    // A Submessage
                    array(
                        'type' => 'submessage',
                        'style' => 'success',
                        'content' => Translations::get('sidebar_widget_help'),
                    ),
                    array(
                        'id' => 'helpie_mp_sidebar1',
                        'type' => 'select',
                        'title' => Translations::get('sidebar1'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => 'sidebars',
                        'default' => 'helpie_sidebar',
                        'dependency' => array('helpie_mp_sidebar_template', 'any', 'both-side-sidebars,left-sidebar,right-sidebar'),
                    ),
                    array(
                        'id' => 'helpie_mp_sidebar2',
                        'type' => 'select',
                        'title' => Translations::get('sidebar2'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => 'sidebars',
                        'default' => 'helpie_sidebar',
                        'dependency' => array('helpie_mp_sidebar_template', '==', 'both-side-sidebars'),
                    ),

                    // A Submessage
                    array(
                        'type' => 'submessage',
                        'style' => 'success',
                        'content' => Translations::get('mp_customize'),
                    ),
                    array(
                        'id' => 'helpie_mp_location',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('mp_location'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'archive' => Translations::get('mp_location_cpt'),
                            'page' => Translations::get('mp_location_page'),
                        ),
                        'default' => 'archive',
                    ),
                    array(
                        'id' => 'helpie_mp_select_page',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('mp_select_page'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => 'pages',
                        'dependency' => array('helpie_mp_location', '==', 'page'),
                        'desc' => Translations::get('mp_select_page_help'),
                        'query_args' => array(
                            'posts_per_page' => -1,
                        ),
                    ),

                    array(
                        'id' => 'helpie_mp_slug',
                        'type' => 'text',
                        'title' => Translations::get('mp_slug'),
                        'default' => 'helpdesk',
                    ),
                    array(
                        'type' => 'content',
                        'content' => Translations::get('mp_link'),
                    ),
                    array(
                        'type' => 'subheading',
                        'title' => Translations::get('hero_section'),
                        'subtitle' => Translations::get('hero_section_subtitle'),
                    ),

                    array(
                        'id' => 'mp_hero_section_order',
                        'type' => 'sortable',
                        'title' => Translations::get('hero_section_control'),
                        'desc' => Translations::get('mp_controls_help'),
                        'fields' => array(
                            array(
                                'id' => 'kb_main_title',
                                'type' => 'text',
                                'title' => Translations::get('kb_title'),
                                'desc' => Translations::get('kb_title_help'),
                            ),

                            array(
                                'id' => 'kb_main_subtitle',
                                'type' => 'text',
                                'title' => Translations::get('kb_subtitle'),
                                'desc' => Translations::get('kb_hero_titles_help'),
                            ),

                            array(
                                'id' => 'main_page_search_display',
                                'type' => 'switcher',
                                'title' => Translations::get('search_display'),
                                'desc' => Translations::get('kb_hero_titles_help'),
                            ),

                        ),
                        'default' => array(
                            'kb_main_title' => 'Helpdesk',
                            'kb_main_subtitle' => 'We’re here to help.',
                            'main_page_search_display' => true,
                        ),
                    ),

                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('mp_meta_data'),
                        'dependency' => array('helpie_mp_location', '==', 'archive'),
                    ),
                    array(
                        'type' => 'submessage',
                        'style' => 'normal',
                        'content' => Translations::get('mp_meta_data_archive'),
                        'dependency' => array('helpie_mp_location', '==', 'archive'),
                    ),
                    array(
                        'id' => 'helpie_mp_meta_title',
                        'type' => 'text',
                        'title' => Translations::get('mp_meta_title'),
                        'dependency' => array('helpie_mp_location', '==', 'archive'),
                        'desc' => Translations::get('mp_meta_title_help'),
                        'default' => 'helpdesk',
                    ),
                    array(
                        'id' => 'helpie_mp_meta_description',
                        'type' => 'text',
                        'title' => Translations::get('mp_meta_desc'),
                        'dependency' => array('helpie_mp_location', '==', 'archive'),
                        'desc' => Translations::get('mp_meta_desc_help'),
                        'default' => 'We are here to help.',
                    ),

                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('mp_components'),
                    ),

                    array(
                        'id' => 'mp_components_order',
                        'type' => 'sortable',
                        'title' => Translations::get('mp_components_control'),
                        'desc' => Translations::get('mp_controls_help'),
                        'fields' => array(
                            array(
                                'id' => 'helpie_mp_show_stats',
                                'type' => 'switcher',
                                'title' => Translations::get('mp_show_stats'),
                                'default' => false,
                            ),
                            array(
                                'id' => 'main_page_categories',
                                'type' => 'switcher',
                                'title' => Translations::get('mp_categories'),
                                'default' => true,
                            ),
                            array(
                                'id' => 'show_article_listing',
                                'type' => 'switcher',
                                'title' => Translations::get('mp_article_listing'),
                                'default' => false,
                            ),
                        ),
                        'default' => array(
                            'helpie_mp_show_stats' => false,
                            'main_page_categories' => true,
                            'show_article_listing' => false,
                        ),
                    ),

                    array(
                        'id' => 'helpie_mp_article_listing',
                        'type' => 'subheading',
                        'content' => Translations::get('article_listing'),
                        'dependency' => array('show_article_listing', '==', 'true'),
                    ),

                    array(
                        'id' => 'helpie_mp_article_listing_title',
                        'type' => 'text',
                        'title' => Translations::get('title'),
                        'default' => 'KB Article Listing',
                        'dependency' => array('show_article_listing', '==', 'true'),
                    ),
                    array(
                        'id' => 'helpie_mp_article_listing_sortby',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('sort_by'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'alphabetical' => Translations::get('alphabetical'),
                            'recent' => Translations::get('recent'),
                            'updated' => Translations::get('recently_updated'),
                            'popular' => Translations::get('popular'),
                        ),
                        'default' => 'recent',
                        'dependency' => array('show_article_listing', '==', 'true'),
                    ),
                    array(
                        'id' => 'helpie_mp_article_listing_topics',
                        'type' => 'select',
                        'chosen' => true,
                        'multiple' => true,
                        'title' => Translations::get('topics'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => 'csf_get_all_helpie_kb_topics',
                        'default' => 'all',
                        'dependency' => array('show_article_listing', '==', 'true'),
                    ),

                    array(
                        'id' => 'helpie_mp_article_listing_style',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('style'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'list' => Translations::get('list'),
                            'card' => Translations::get('card'),
                        ),
                        'default' => 'list',
                        'dependency' => array('show_article_listing', '==', 'true'),
                    ),

                    array(
                        'id' => 'helpie_mp_article_listing_num_of_cols',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('num_of_cols'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'one' => 1,
                            'two' => 2,
                            'three' => 3,
                            'four' => 4,
                        ),
                        'default' => 'three',
                        'dependency' => array('show_article_listing|helpie_mp_article_listing_style', '==|==', 'true|card'),
                    ),
                    array(
                        'id' => 'helpie_mp_article_listing_show_image',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('show_image'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'true' => 'True',
                            'false' => 'False',
                        ),
                        'default' => 'true',
                        'dependency' => array('show_article_listing|helpie_mp_article_listing_style', '==|==', 'true|card'),
                    ),

                    array(
                        'id' => 'helpie_mp_article_listing_show_extra',
                        'title' => Translations::get('show_extra_info'),
                        'chosen' => true,
                        'options' => array(
                            'true' => Translations::get('true'),
                            'false' => Translations::get('false'),
                        ),
                        'type' => 'select',
                        'default' => 'true',

                        'dependency' => array('show_article_listing|helpie_mp_article_listing_style', '==|==', 'true|card'),
                    ),

                    array(
                        'id' => 'helpie_mp_article_listing_limit',
                        'type' => 'text',
                        'title' => Translations::get('limit'),
                        'default' => 5,
                        'validate' => 'csf_validate_numeric',
                        'dependency' => array('show_article_listing', '==', 'true'),
                    ),

                    array(
                        'id' => 'mp_categories_settings',
                        'type' => 'subheading',
                        'content' => Translations::get('category_listing'),
                        'dependency' => array('main_page_categories', '==', 'true'),
                    ),
                    array(
                        'id' => 'helpie_mp_template',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('mp_category_listing_style'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'boxed' => Translations::get('boxed'),
                            'boxed1' => Translations::get('boxed1'),
                            'modern' => Translations::get('modern'),
                        ),
                        'default' => 'boxed',
                        'dependency' => array('main_page_categories', '==', 'true'),
                    ),
                    array(
                        'id' => 'helpie_mp_boxed_description',
                        'type' => 'switcher',
                        'title' => Translations::get('show_description'),
                        'dependency' => array('main_page_categories', '==', 'true'),
                        'default' => false,
                    ),
                    array(
                        'id' => 'category_listing_graphic_type',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('image_or_icon'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'image' => Translations::get('image'),
                            'icon' => Translations::get('icon'),
                        ),
                        'default' => 'image',
                        'desc' => Translations::get('mp_category_listing_graphic_help'),
                        'dependency' => array('main_page_categories', '==', 'true'),
                    ),
                    array(
                        'id' => 'category_listing_children_type',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('children_type'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'none' => Translations::get('dont_show_children'),
                            'articles' => Translations::get('articles'),
                            'sub-categories' => Translations::get('sub_categories'),
                        ),
                        'default' => 'articles',
                        'dependency' => array('main_page_categories|helpie_mp_template', '==|!=', 'true|boxed'),
                    ),
                    array(
                        'id' => 'helpie_mp_cl_cols',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('num_of_cols'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'one' => '1',
                            'two' => '2',
                            'three' => '3',
                            'four' => '4',
                        ),
                        'default' => 'three',
                        'dependency' => array('main_page_categories', '==', 'true'),
                    ),
                    array(
                        'id' => 'helpie_mp_no_cat_articles',
                        'type' => 'text',
                        'title' => Translations::get('mp_no_cat_articles'),
                        'default' => 5,
                        'validate' => 'csf_validate_numeric',
                        'dependency' => array('main_page_categories', '==', 'true'),
                    ),
                    array(
                        'id' => 'helpie_mp_cats',
                        'type' => 'sorter',
                        'title' => Translations::get('mp_cats'),
                        'default' => \get_sorter_terms(),
                        'enabled_title' => Translations::get('enabled'),
                        'disabled_title' => Translations::get('disabled'),
                    ),

                ),
            ));
        }

        public function search($prefix)
        {

            $search_single_item_selectors = [
                '.helpie-main-content-area .helpie-search-listing .helpie-element.item',
                '.helpie-main-content-area .helpie-search-listing .helpie-element.item:first-child',
            ];

            $search_page_id = get_option('helpdesk_search_page_id');

            // Create a section
            \CSF::createSection($prefix, array(
                'title' => Translations::get('search_settings'),
                'icon' => 'fa fa-search',
                'fields' => array(
                    array(
                        'id' => 'helpie_search_page',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('search_select_page'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => 'pages',
                        'default' => $search_page_id,
                        'desc' => Translations::get('search_select_page_help'),
                        'query_args' => array(
                            'posts_per_page' => -1,
                        ),
                    ),
                    array(
                        'id' => 'helpie_search_placeholder_text',
                        'type' => 'text',
                        'title' => Translations::get('search_placeholder_txt'),
                        'default' => 'What can I help you with?',
                    ),

                    array(
                        'id' => 'search_no_query_text',
                        'type' => 'text',
                        'title' => Translations::get('search_no_query_txt'),
                        'default' => 'Please search something !',
                    ),

                    array(
                        'id' => 'empty_search_result_label',
                        'type' => 'text',
                        'title' => Translations::get('search_empty_result_label'),
                        'default' => 'Did not match any articles !',
                    ),

                    array(
                        'id' => 'helpie_search_page_featured_image_show',
                        'type' => 'switcher',
                        'title' => Translations::get('search_page_featured_image_show'),
                        'default' => true,
                    ),

                    array(
                        'id' => 'helpie_search_page_meta_data_show',
                        'type' => 'switcher',
                        'title' => Translations::get('search_page_meta_data_show'),
                        'default' => true,
                    ),

                    array(
                        'id' => 'helpie_search_page_description_show',
                        'type' => 'switcher',
                        'title' => Translations::get('search_page_description_show'),
                        'default' => true,
                    ),

                    array(
                        'id' => 'helpie_search_page_tags_show',
                        'type' => 'switcher',
                        'title' => Translations::get('search_page_tags_show'),
                        'default' => true,
                    ),

                    array(
                        'id' => 'helpie_search_page_description_length',
                        'type' => 'text',
                        'title' => Translations::get('search_page_desc_length'),
                        'default' => 200,
                        'dependency' => array('helpie_search_page_description_show', '==', '1'),
                    ),

                    array(
                        'type' => 'submessage',
                        'style' => 'success',
                        'content' => Translations::get('search_page_desc_length_help'),
                        'dependency' => array('helpie_search_page_description_show', '==', '1'),
                    ),

                    /* Design */
                    array(
                        'type' => 'subheading',
                        'title' => strtoupper(Translations::get('style')),
                        'subtitle' => Translations::get('search_style_help'),
                    ),

                    array(
                        'id' => 'search-header-typography',
                        'type' => 'typography',
                        'title' => Translations::get('search_header_typography'),
                        'output' => array('.helpie-search-listing .helpie-element .item-content .header'),
                        'output_important' => true,
                        'text_align' => false,
                    ),

                    array(
                        'id' => 'search-title-typography',
                        'type' => 'typography',
                        'title' => Translations::get('search_title_typography'),
                        'output' => array('.helpie-search-listing .item-content .header .item-title'),
                        'output_important' => true,
                        'text_align' => false,
                    ),

                    array(
                        'id' => 'search-cat-typography',
                        'type' => 'typography',
                        'title' => Translations::get('search_category_typography'),
                        'output' => array('.helpie-search-listing .helpie-element .item-content .header .item-cat_name'),
                        'output_important' => true,
                        'text_align' => false,
                    ),

                    array(
                        'id' => 'search-meta-typography',
                        'type' => 'typography',
                        'title' => Translations::get('search_meta_typography'),
                        'output' => array('.helpie-search-listing .helpie-element .item-content .meta'),
                        'output_important' => true,
                        'text_align' => false,
                    ),

                    array(
                        'id' => 'search-text-typography',
                        'type' => 'typography',
                        'title' => Translations::get('search_text_typography'),
                        'output' => array('.helpie-search-listing .helpie-element .item-content .description p'),
                        'output_important' => true,
                        'text_align' => false,
                    ),

                    array(
                        'id' => 'helpiekb_search_results_single_border',
                        'type' => 'border',
                        'title' => Translations::get('border'),
                        'output' => $search_single_item_selectors,
                        'output_important' => true,
                    ),

                    array(
                        'id' => 'helpiekb_search_results_single_padding',
                        'type' => 'spacing',
                        'title' => Translations::get('padding'),
                        'output' => $search_single_item_selectors,
                        'output_mode' => 'padding',
                        'output_important' => true,
                    ),

                    array(
                        'id' => 'helpiekb_search_results_single_margin',
                        'type' => 'spacing',
                        'title' => Translations::get('margin'),
                        'output' => $search_single_item_selectors,
                        'output_mode' => 'margin',
                        'output_important' => true,
                    ),

                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('search_box'),
                    ),
                    array(
                        'id' => 'helpie_show_search_border',
                        'type' => 'switcher',
                        'title' => Translations::get('search_border_display'),
                        'default' => false,
                    ),

                    array(
                        'id' => 'helpie_search_border_color',
                        'type' => 'color',
                        'title' => Translations::get('search_border_color'),
                        'default' => false,
                    ),

                    array(
                        'id' => 'helpie_search_border_style',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('search_border_style'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'rounded' => Translations::get('rounded'),
                            'semi-rounded' => Translations::get('semi_rounded'),
                            'squared' => Translations::get('squared'),
                        ),
                        'default' => 'semi-rounded',
                    ),

                    // array(
                    //     'id'    => 'autosuggest-title-typography',
                    //     'type'  => 'typography',
                    //     'title' => 'Autosuggest Title Typography',
                    // ),
                    // array(
                    //     'id'    => 'autosuggest-category-typography',
                    //     'type'  => 'typography',
                    //     'title' => 'Autosuggest Category Typography',
                    // ),
                    // array(
                    //     'id'    => 'autosuggest-text-typography',
                    //     'type'  => 'typography',
                    //     'title' => 'Autosuggest Text Typography',
                    // ),
                ),
            ));
        }

        public function frontend_editor($prefix)
        {
            // Create a section
            \CSF::createSection($prefix, array(
                'title' => Translations::get('frontend_editor'),
                'icon' => 'fa fa-font',
                'fields' => array(

                    0 => array(
                        'id' => 'kb_frontend_enable',
                        'type' => 'switcher',
                        'title' => Translations::get('enable_editor'),
                        'default' => false,
                    ),
                    1 => array(
                        'id' => 'kb_num_of_revisions',
                        'type' => 'text',
                        'title' => Translations::get('num_of_revision'),
                        'default' => 20,
                        'validate' => 'csf_validate_numeric',
                        'dependency' => array('kb_frontend_enable', '==', '1'),
                    ),
                    2 => array(
                        'id' => 'kb_editor_type',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('editor_type'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'inline' => Translations::get('inline_editor'),
                            'wpeditor' => Translations::get('tinymce_editor'),
                        ),
                        'default' => 'inline',
                        'dependency' => array('kb_frontend_enable', '==', '1'),
                    ),

                ),

            ));
        }

        public function password_protect($prefix)
        {

            // Create a section

            \CSF::createSection($prefix, array(

                'title' => Translations::get('password_settings'),
                'icon' => 'fa fa-unlock-alt',
                'fields' => array(
                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('password_settings'),
                    ),
                    // A Submessage
                    array(
                        'type' => 'submessage',
                        'style' => 'info',
                        'content' => Translations::get('pp_not_required'),
                    ),
                    array(

                        'id' => 'helpie_password_options',
                        'type' => 'repeater',
                        'title' => Translations::get('pp_plural'),
                        'fields' => array(
                            array(
                                'id' => 'password_for_content',
                                'type' => 'select',
                                'chosen' => true,
                                'multiple' => true,
                                'title' => Translations::get('category'),
                                'placeholder' => Translations::get('select_an_option'),
                                'options' => 'csf_get_all_helpie_kb_mp_topics',
                                'default' => 'all',
                            ),
                            array(
                                'id' => 'password_for',
                                'type' => 'text',
                                'title' => Translations::get('pp_singular'),
                                'attributes' => array(
                                    'type' => 'password',
                                ),
                            ),
                        ),
                    ),

                    array(
                        'id' => 'helpie_password_remember_days',
                        'type' => 'text',
                        'title' => Translations::get('pp_remember_days'),
                        'default' => 7,
                        'validate' => 'csf_validate_numeric',
                    ),
                ),
            ));
        }

        public function styles($prefix)
        {
            // Create a section
            \CSF::createSection(
                $prefix,
                array(
                    'title' => Translations::get('design'),
                    'icon' => 'fa fa-magic',
                    'fields' => array(

                        array(
                            'id' => 'design',
                            'type' => 'tabbed',
                            'tabs' => array(
                                array(
                                    'title' => Translations::get('layout'),
                                    'icon' => 'fa fa-columns',
                                    'fields' => $this->layout($prefix),
                                ),
                                array(
                                    'title' => Translations::get('typography'),
                                    'icon' => 'fa fa-align-center',
                                    'fields' => $this->typography($prefix),
                                ),
                                array(
                                    'title' => Translations::get('hero_section'),
                                    'icon' => 'fa fa-building',
                                    'fields' => $this->hero_section_design(),
                                ),
                                // array(
                                //     'title'     => 'Search',
                                //     'icon'      => 'fa fa-search',
                                //     'fields'    => $this->search_design()
                                // ),
                            ),
                        ),
                    ),
                )

            );
        }

        public function layout($prefix)
        {
            return array(
                array(
                    'type' => 'subheading',
                    'content' => Translations::get('layout'),
                ),

                array(
                    'id' => 'helpiekb-wrapper-width',
                    'type' => 'dimensions',
                    'title' => Translations::get('wrapper_width'),
                    'default' => array(
                        'width' => '980',
                        'unit' => 'px',
                    ),
                    'height' => false,
                    'output' => "#helpiekb-main-wrapper",
                ),

                array(
                    'type' => 'subheading',
                    'content' => Translations::get('kb_content_area'),
                ),
                array(
                    'id' => 'helpie_margin_top_desktop',
                    'type' => 'text',
                    'title' => Translations::get('margin_top_desktop'),
                    'default' => 0,
                    'validate' => 'csf_validate_numeric',
                ),

                array(
                    'id' => 'helpie_margin_top_tablet',
                    'type' => 'text',
                    'title' => Translations::get('margin_top_tablet'),
                    'default' => 0,
                    'validate' => 'csf_validate_numeric',
                ),
                array(
                    'id' => 'helpie_margin_top_mobile',
                    'type' => 'text',
                    'title' => Translations::get('margin_top_mobile'),
                    'default' => 0,
                    'validate' => 'csf_validate_numeric',
                ),
            );
        }

        public function hero_section_design()
        {
            return array(
                array(
                    'type' => 'subheading',
                    'content' => Translations::get('hero_section'),
                ),

                array(
                    'id' => 'helpie_wa_background_type',
                    'type' => 'select',
                    'chosen' => true,
                    'placeholder' => Translations::get('select_an_option'),
                    'title' => Translations::get('hero_section'),
                    'options' => array(
                        'single-color' => Translations::get('primary_color'),
                        'color-gradient' => Translations::get('color_gradient'),
                        'background-image' => Translations::get('background_image'),
                        'gradient-plus-background-image' => Translations::get('gradient_plus_background_image'),
                    ),
                    'default' => 'single-color',
                ),

                array(
                    'id' => 'helpie_brand_primary_color',
                    'type' => 'color',
                    'title' => Translations::get('primary_brand_color'),
                    'default' => '#f4f3f3',
                    'output' => '.helpie_helpdesk',
                    'output_mode' => 'background-color',
                    'output_important' => true,
                ),

                array(
                    'id' => 'helpie_wa_image',
                    'title' => Translations::get('heror_area_image'),
                    'type' => 'media',
                    'title' => 'Media',
                    'library' => 'image',
                    'dependency' => array('helpie_wa_background_type', '==', 'background-image'),
                ),

                array(
                    'id' => 'helpie_wa_illustration',
                    'type' => 'image_select',
                    'title' => Translations::get('heror_area_illustration'),
                    'options' => array(
                        'snowy-mountain' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_wa_illustration/snowy-mountain.png',
                        'team-space' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_wa_illustration/team-space.png',
                        'books' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_wa_illustration/books.png',
                        'bubbles' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_wa_illustration/bubbles.png',
                    ),
                    'default' => 'team-space',
                    'dependency' => array('helpie_wa_background_type', '==', 'gradient-plus-background-image'),
                ),

                array(
                    'id' => 'helpie_wa_gradient1',
                    'type' => 'color',
                    'title' => Translations::get('heror_area_gradient1'),
                    'default' => '#777777',
                    'dependency' => array('helpie_wa_background_type', '==', 'color-gradient'),
                ),

                array(
                    'id' => 'helpie_wa_gradient2',
                    'type' => 'color',
                    'title' => Translations::get('heror_area_gradient2'),
                    'default' => '#777777',
                    'dependency' => array('helpie_wa_background_type', '==', 'color-gradient'),
                ),

                array(
                    'id' => 'helpie_brand_title_color',
                    'type' => 'color',
                    'title' => Translations::get('heror_area_title_color'),
                    'default' => '#03363d',
                ),

                array(
                    'id' => 'helpie_wa_text_color',
                    'type' => 'color',
                    'title' => Translations::get('heror_area_text_color'),
                    'default' => '#03363d',
                ),

                array(
                    'id' => 'helpiekb_wa_padding',
                    'type' => 'spacing',
                    'title' => Translations::get('heror_area_section_padding'),
                    'output' => '.helpie_helpdesk',
                    'output_mode' => 'padding',
                    'output_important' => true,
                ),
            );
        }

        public function typography($prefix)
        {
            return array(
                array(
                    'type' => 'subheading',
                    'content' => Translations::get('kb_typography'),
                ),
                array(
                    'id' => 'h1-typography',
                    'type' => 'typography',
                    'title' => Translations::get('h1_typography'),
                    'output' => '.helpie-single-page-module h1',
                    'output_important' => true,
                ),
                array(
                    'id' => 'h2-typography',
                    'type' => 'typography',
                    'title' => Translations::get('h2_typography'),
                    'output' => array('.helpie-single-page-module h2'),
                    'output_important' => true,
                ),
                array(
                    'id' => 'h3-typography',
                    'type' => 'typography',
                    'title' => Translations::get('h3_typography'),
                    'output' => array('.helpie-single-page-module h3'),
                    'output_important' => true,
                ),
                array(
                    'id' => 'h4-typography',
                    'type' => 'typography',
                    'title' => Translations::get('h4_typography'),
                    'output' => array('.helpie-single-page-module h4'),
                    'output_important' => true,
                ),
                array(
                    'id' => 'h5-typography',
                    'type' => 'typography',
                    'title' => Translations::get('h5_typography'),
                    'output' => array('.helpie-single-page-module h5'),
                    'output_important' => true,
                ),
                array(
                    'id' => 'h6-typography',
                    'type' => 'typography',
                    'title' => Translations::get('h6_typography'),
                    'output' => array('.helpie-single-page-module h6'),
                    'output_important' => true,
                ),
                array(
                    'id' => 'p-typography',
                    'type' => 'typography',
                    'title' => Translations::get('ptag_typography'),
                    'output' => array('.helpie-single-page-module p'),
                    'output_important' => true,
                ),

                // TODO : Not working. Check
                // array(
                //     'id'    => 'link-color',
                //     'type'  => 'link_color',
                //     'title' => 'Link Color',
                //     'output' => array('.helpie-single-page-module a'),
                //     'output_important' => true,
                // ),

            );
        }
        public function dynamic_capabilities($prefix)
        {
            $cap_fields = $this->get_cap_fields('global');
            $cap_fields_can_publish = $this->get_cap_fields('global', 'can_publish'); // this is default value for edit, publish, approve.

            \CSF::createSection($prefix, array(
                'title' => Translations::get('dynamic_capabilities'),
                'icon' => 'fa fa-users',
                'fields' => array(
                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('dynamic_capabilities'),
                    ),
                    // A Submessage
                    array(
                        'type' => 'submessage',
                        'style' => 'info',
                        'content' => Translations::get('dc_non_admin_restriction'),
                    ),

                    array(
                        'type' => 'submessage',
                        'style' => 'success',
                        'content' => Translations::get('dc_non_admin_restriction_help'),
                    ),
                    array(
                        'id' => 'helpie_dynamic_capability',
                        'type' => 'tabbed',
                        'tabs' => array(
                            array(
                                'title' => Translations::get('can_view'),
                                'icon' => 'fa fa-eye',
                                'fields' => array(
                                    array(
                                        'id' => 'can_view',
                                        'type' => 'fieldset',
                                        'title' => Translations::get('who_can_view'),
                                        'fields' => $cap_fields,
                                    ),
                                ),
                            ),

                            array(
                                'title' => Translations::get('can_edit'),
                                'icon' => 'fa fa-pencil',
                                'fields' => array(
                                    array(
                                        'id' => 'can_edit',
                                        'type' => 'fieldset',
                                        'title' => Translations::get('who_can_edit'),
                                        'fields' => $cap_fields_can_publish,
                                    ),
                                ),
                            ),

                            array(
                                'title' => Translations::get('can_publish'),
                                'icon' => 'fa fa-book',
                                'fields' => array(
                                    array(
                                        'id' => 'can_publish',
                                        'type' => 'fieldset',
                                        'title' => Translations::get('who_can_publish'),
                                        'fields' => $cap_fields_can_publish,
                                    ),
                                ),
                            ),

                            array(
                                'title' => Translations::get('can_approve'),
                                'icon' => 'fa fa-check',
                                'fields' => array(
                                    array(
                                        'id' => 'can_approve',
                                        'type' => 'fieldset',
                                        'title' => Translations::get('who_can_approve'),
                                        'fields' => $cap_fields_can_publish,
                                    ),
                                ),
                            ),

                        ),
                    ),
                ),
            ));
        }

        public function components($prefix)
        {
            // Create a section
            \CSF::createSection(
                $prefix,
                array(
                    'title' => Translations::get('components'),
                    'icon' => 'fa fa-inbox',
                    'fields' => array(

                        array(
                            'id' => 'helpie_breadcrumbs',
                            'type' => 'switcher',
                            'title' => Translations::get('breadcrumbs'),
                            'default' => true,
                        ),
                        array(
                            'id' => 'article_order',
                            'type' => 'select',
                            'title' => Translations::get('article_order'),
                            'chosen' => true,
                            'placeholder' => Translations::get('select_an_option'),
                            'options' => array(
                                'alphabetical' => Translations::get('alphabetical'),
                                'menu_order' => Translations::get('menu_order'),
                                'asc_post_date' => Translations::get('asc_post_date'),
                                'desc_post_date' => Translations::get('desc_post_date'),
                            ),
                            'default' => 'menu_order',
                        ),

                    ),
                )
            );
        }

        public function demo_import($prefix)
        {
            $entries = get_option('helpiekb_imported_entries');
            if ($this->isset_entries($entries)) {
                $demo_section = $this->extras->get_demo_section($entries);
                // Create a section
                \CSF::createSection(
                    $prefix,
                    array(
                        'title' => Translations::get('demo_import'),
                        'icon' => 'fa fa-gift',
                        'fields' => array(

                            array(
                                'type' => 'content',
                                'content' => $demo_section,
                            ),
                        ),
                    )
                );
            }
        }

        protected function isset_entries($entries)
        {
            $isset = false;
            if (!empty($entries)) {
                foreach ($entries as $type) {
                    if (isset($type) && !empty($type)) {
                        $isset = true;
                    }
                }
            }

            return $isset;
        }

        public function backup($prefix)
        {
            // Create a section
            \CSF::createSection(
                $prefix,
                array(
                    'title' => Translations::get('backup_restore'),
                    'icon' => 'fa fa-download',
                    'fields' => array(
                        array(
                            'type' => 'subheading',
                            'content' => Translations::get('backup_restore_settings'),
                            'subtitle' => Translations::get('backup_restore_settings_sub'),
                        ),
                        // A Submessage
                        array(
                            'type' => 'submessage',
                            'style' => 'success',
                            'content' => Translations::get('backup_restore_settings_help'),
                        ),
                        array(
                            'type' => 'backup',
                        ),
                    ),
                )
            );
        }

        public function performance($prefix)
        {
            \CSF::createSection($prefix, array(
                'title' => Translations::get('performance'),
                'icon' => 'fa fa-gear',
                'fields' => array(
                    array(
                        'type' => 'submessage',
                        'style' => 'warning',
                        'content' => Translations::get('performance_warning'),
                    ),
                ),

            ));
        }

        public function toc($prefix)
        {
            // Create a section
            \CSF::createSection($prefix, array(
                'title' => Translations::get('toc'),
                'icon' => 'fa fa-bars',
                'fields' => array(

                    array(
                        'id' => 'helpie_sidebar_title',
                        'type' => 'text',
                        'title' => Translations::get('toc_title'),
                        'default' => 'Table of Contents',
                    ),
                    array(
                        'id' => 'helpie_sidebar_fixed',
                        'type' => 'switcher',
                        'title' => Translations::get('toc_fixed_sidebar'),
                        'default' => false,
                    ),
                    array(
                        'id' => 'helpie_sidebar_type',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('toc_sidebar_type'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'full-nav' => Translations::get('toc_full_nav'),
                            'page-scroll-only' => Translations::get('toc_scroll_only'),
                        ),
                        'default' => 'full-nav',
                        'desc' => Translations::get('toc_sidebar_type_help'),
                    ),
                    array(
                        'type' => 'subheading',
                        'title' => Translations::get('toc_full_nav_settings'),
                        'subtitle' => Translations::get('toc_full_nav_settings_sub'),
                        'dependency' => array('helpie_sidebar_type', '==', 'full-nav'),
                    ),
                    array(
                        'id' => 'helpie_sidebar_auto_toc',
                        'type' => 'switcher',
                        'title' => Translations::get('toc_display_in_page_nav'),
                        'desc' => Translations::get('toc_display_in_page_nav_help'),
                        'dependency' => array('helpie_sidebar_type', '==', 'full-nav'),
                        'default' => false,
                    ),

                    array(
                        'id' => 'helpie_sidebar_categories',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('toc_sidebar_categories'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'all' => Translations::get('all'),
                            'mp' => Translations::get('main_page'),
                        ),
                        'default' => 'all',
                        'dependency' => array('helpie_sidebar_type', '==', 'full-nav'),
                        'desc' => Translations::get('toc_sidebar_categories_help'),
                    ),
                    // array(
                    //     'id' => 'helpie_sidebar_cat_toggle',
                    //     'type' => 'switcher',
                    //     'title' => Translations::get('toc_cat_toggle'),
                    //     'default' => true,
                    //     'dependency' => array('helpie_sidebar_type', '==', 'full-nav'),
                    // ),
                    array(
                        'id' => 'helpie_sidebar_category_anchor_link',
                        'type' => 'switcher',
                        'title' => Translations::get('toc_category_anchor_link'),
                        'default' => false,
                        'dependency' => array('helpie_sidebar_type', '==', 'full-nav'),
                    ),
                    array(
                        'id' => 'helpie_sidebar_num_of_child_category',
                        'type' => 'text',
                        'title' => Translations::get('toc_display_num_of_child_category'),
                        'default' => 5,
                        'validate' => 'csf_validate_numeric',
                        'dependency' => array('helpie_sidebar_type', '==', 'full-nav'),
                    ),
                    array(
                        'id' => 'helpie_sidebar_num_of_articles',
                        'type' => 'text',
                        'title' => Translations::get('toc_display_num_of_articles'),
                        'default' => 5,
                        'validate' => 'csf_validate_numeric',
                        'dependency' => array('helpie_sidebar_type', '==', 'full-nav'),
                    ),

                    array(
                        'id' => 'helpie_sidebar_show_articles',
                        'type' => 'switcher',
                        'title' => Translations::get('toc_display_articles'),
                        'default' => true,
                        'dependency' => array('helpie_sidebar_type', '==', 'full-nav'),
                    ),

                    array(
                        'type' => 'submessage',
                        'style' => 'warning',
                        'content' => Translations::get('performance_warning'),
                    ),
                    array(
                        'type' => 'subheading',
                        'title' => Translations::get('toc_in_page_nav_settings'),
                        'subtitle' => Translations::get('toc_in_page_nav_settings_sub'),
                    ),

                    array(
                        'id' => 'helpie_auto_toc_exclude_headings',
                        'type' => 'select',
                        'multiple' => true,
                        'chosen' => true,
                        'title' => Translations::get('toc_exclude_headings'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            '1' => 'h1',
                            '2' => 'h2',
                            '3' => 'h3',
                            '4' => 'h4',
                            '5' => 'h5',
                            '6' => 'h6',
                        ),
                        'desc' => Translations::get('toc_exclude_headings_help'),
                    ),
                    array(
                        'id' => 'helpie_auto_toc_title',
                        'type' => 'text',
                        'title' => Translations::get('toc_in_page_nav_title'),
                        'default' => 'In This Article',
                    ),
                    array(
                        'id' => 'helpie_auto_toc_bullet',
                        'type' => 'switcher',
                        'title' => Translations::get('toc_numeric_bullet'),
                        'default' => false,
                    ),
                    array(
                        'id' => 'helpie_auto_toc_section_page_url',
                        'type' => 'switcher',
                        'title' => Translations::get('toc_section_page_url'),
                        'default' => false,
                    ),
                    array(
                        'id' => 'helpie_auto_toc_section_page_url_text',
                        'type' => 'text',
                        'title' => Translations::get('toc_section_page_url_text'),
                        'default' => 'helpie-sp',
                    ),
                    array(
                        'id' => 'helpie_auto_toc_back_to_top_link',
                        'type' => 'switcher',
                        'title' => Translations::get('toc_back_to_top_link'),
                        'default' => false,
                    ),
                    array(
                        'id' => 'helpie_auto_toc_back_to_top_text',
                        'type' => 'text',
                        'title' => Translations::get('toc_back_to_top_text'),
                        'default' => 'Back to Top',
                        'dependency' => array('helpie_auto_toc_back_to_top_link', '==', 'true'),

                    ),
                    array(
                        'id' => 'helpie_auto_toc_scroll_back_to_site_top',
                        'type' => 'switcher',
                        'title' => Translations::get('toc_scroll_back_to_site_top'),
                        'dependency' => array('helpie_auto_toc_back_to_top_link', '==', 'true'),
                        'desc' => __('by default, back to top scrolls to article text top', HELPIE_DOMAIN),
                        'default' => false,
                    ),
                    array(
                        'id' => 'helpie_auto_toc_smooth_scroll',
                        'type' => 'switcher',
                        'title' => Translations::get('toc_smooth_scroll'),
                        'default' => true,
                    ),

                ),

            ));
        }

        public function single_page($prefix)
        {
            // SECTION: Single Article

            \CSF::createSection($prefix, array(
                'title' => Translations::get('single_page'),
                'icon' => 'fa fa-file',

                'fields' => array(
                    array(
                        'type' => 'subheading',
                        'title' => Translations::get('sp_template'),
                        'subtitle' => Translations::get('sp_template_help'),
                    ),
                    // A Submessage
                    array(
                        'type' => 'submessage',
                        'style' => 'success',
                        'content' => Translations::get('sp_template_source_help'),
                    ),
                    array(
                        'id' => 'helpie_sp_template_source',
                        'type' => 'select',
                        'title' => Translations::get('sp_template_source'),
                        'placeholder' => Translations::get('select_an_option'),
                        'chosen' => true,
                        'options' => array(
                            'helpie' => Translations::get('sp_helpie_template'),
                            'elementor' => Translations::get('sp_theme_template'),
                        ),
                        'default' => 'helpie',
                    ),
                    // A Submessage
                    array(
                        'type' => 'submessage',
                        'style' => 'success',
                        'content' => Translations::get('sidebar_widget_help'),
                    ),

                    // A Submessage
                    array(
                        'type' => 'submessage',
                        'style' => 'warning',
                        'content' => Translations::get('sidebar_widget_toc_help'),
                        'dependency' => array('helpie_sp_template_source', '==', 'elementor'),
                    ),

                    array(
                        'id' => 'helpie_sp_template',
                        'type' => 'image_select',
                        'title' => Translations::get('template'),
                        'options' => array(
                            'both-side-sidebars' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/both-side-sidebars.png',
                            'left-sidebar' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/left-sidebar.png',
                            'right-sidebar' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/right-sidebar.png',
                            'full-width' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/full-width.png',
                        ),
                        'default' => 'left-sidebar',
                        'dependency' => array('helpie_sp_template_source', '==', 'helpie'),
                    ),

                    array(
                        'id' => 'helpie_sp_sidebar1',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('sidebar1'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => 'sidebars',
                        'default' => 'helpie_sidebar',
                        'dependency' => array('helpie_sp_template|helpie_sp_template_source', 'any|==', 'both-side-sidebars,left-sidebar,right-sidebar|helpie'),
                    ),
                    array(
                        'id' => 'helpie_sp_sidebar2',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('sidebar2'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => 'sidebars',
                        'default' => 'helpie_sidebar',
                        'dependency' => array('helpie_sp_template|helpie_sp_template_source', '==|==', 'both-side-sidebars|helpie'),
                    ),
                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('sp_labelling'),
                    ),
                    array(
                        'id' => 'helpie_sp_cpt_label',
                        'type' => 'text',
                        'title' => Translations::get('sp_cpt_label'),
                        'default' => __('Article', HELPIE_DOMAIN),
                        'help' => Translations::get('sp_cpt_label_help'),
                    ),

                    array(
                        'id' => 'helpie_sp_cpt_label_plural',
                        'type' => 'text',
                        'title' => Translations::get('sp_cpt_label_plural'),
                        'default' => __('Articles', HELPIE_DOMAIN),
                    ),

                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('components'),
                    ),

                    array(
                        'id' => 'helpie_single_page_search_display',
                        'type' => 'switcher',
                        'title' => Translations::get('search_display'),
                        'default' => true,
                    ),

                    array(
                        'id' => 'helpie_single_page_updatedby_display',
                        'type' => 'switcher',
                        'title' => Translations::get('sp_display_author_name'),
                        'default' => true,
                    ),
                    array(
                        'id' => 'helpie_single_page_updatedon_display',
                        'type' => 'switcher',
                        'title' => Translations::get('sp_display_updatedon'),
                        'default' => false,
                    ),
                    array(
                        'id' => 'helpie_single_page_show_pageviews',
                        'type' => 'switcher',
                        'title' => Translations::get('sp_display_pageviews'),
                        'default' => false,
                    ),

                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('voting'),
                    ),

                    array(
                        'id' => 'helpie_voting_template',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('voting_system'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'classic' => Translations::get('classic'),
                            'emotion' => Translations::get('emotion'),
                            'none' => Translations::get('none'),
                        ),
                        'default' => 'emotion',
                    ),

                    array(
                        'id' => 'helpie_voting_label',
                        'type' => 'text',
                        'title' => Translations::get('voting_label'),
                        'default' => 'How did you like this article?',
                    ),

                    array(
                        'id' => 'helpie_voting_access',
                        'type' => 'switcher',
                        'title' => Translations::get('voting_access'),
                        'dependency' => array('helpie_voting_template', '!=', 'none'),
                        'default' => false,
                    ),

                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('comments'),
                        'dependency' => array('helpie_sp_template_source', '==', 'helpie'),
                    ),

                    array(
                        'id' => 'helpie_show_comments',
                        'type' => 'switcher',
                        'title' => Translations::get('show_comments'),
                        'default' => false,
                        'dependency' => array('helpie_sp_template_source', '==', 'helpie'),
                    ),
                    array(
                        'type' => 'subheading',
                        'content' => Translations::get('style'),
                        'dependency' => array('helpie_sp_template_source', '==', 'helpie'),
                    ),
                    array(
                        'id' => 'helpie_post_title_color',
                        'type' => 'color',
                        'title' => Translations::get('sp_title_color'),
                        'default' => '#777777',
                        'dependency' => array('helpie_sp_template_source', '==', 'helpie'),
                    ),
                ),
            ));
        }

        public function category_page($prefix)
        {
            // Create a section
            \CSF::createSection($prefix, array(
                'title' => Translations::get('category_page'),
                'icon' => 'fa fa-folder-open',
                'fields' => array(
                    array(
                        'id' => 'helpie_cp_template',
                        'type' => 'image_select',
                        'title' => Translations::get('template'),
                        'options' => array(
                            'both-side-sidebars' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/both-side-sidebars.png',
                            'left-sidebar' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/left-sidebar.png',
                            'right-sidebar' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/right-sidebar.png',
                            'full-width' => HELPIE_PLUGIN_URL . '/includes/asset-files/images/helpie_cp_template/full-width.png',
                        ),
                        'default' => 'left-sidebar',
                    ),
                    // A Submessage
                    array(
                        'type' => 'submessage',
                        'style' => 'success',
                        'content' => Translations::get('sidebar_widget_help'),
                    ),
                    array(
                        'id' => 'helpie_cp_sidebar1',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('sidebar1'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => 'sidebars',
                        'default' => 'helpie_sidebar',
                        'dependency' => array('helpie_cp_template', 'any', 'both-side-sidebars,left-sidebar,right-sidebar'),
                    ),
                    array(
                        'id' => 'helpie_cp_sidebar2',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('sidebar2'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => 'sidebars',
                        'default' => 'helpie_sidebar',
                        'dependency' => array('helpie_cp_template', '==', 'both-side-sidebars'),
                    ),

                    array(
                        'id' => 'helpie_cat_page_search_display',
                        'type' => 'switcher',
                        'title' => Translations::get('search_display'),
                        'default' => true,
                    ),
                    array(
                        'id' => 'helpie_cp_slug',
                        'type' => 'text',
                        'title' => Translations::get('cp_slug'),
                        'default' => 'helpdesk_category',
                    ),
                    array(
                        'id' => 'helpie_cat_title_color',
                        'type' => 'color',
                        'title' => Translations::get('cp_title_color'),
                        'default' => '#777777',
                        'output' => array('.helpie-single-page-module.category-page h1'),
                        'output_important' => true,
                    ),
                    array(
                        'type' => 'subheading',
                        'title' => Translations::get('article_listing'),
                        'subtitle' => '',
                    ),
                    array(
                        'id' => 'helpie_cp_article_list_style',
                        'type' => 'select',
                        'title' => Translations::get('al_style'),
                        'options' => array(
                            'minimal' => Translations::get('minimal'),
                            'boxed' => Translations::get('boxed'),
                        ),
                        'default' => 'boxed',
                    ),

                    array(
                        'id' => 'helpie_cp_article_list_columns',
                        'type' => 'select',
                        'title' => Translations::get('al_no_of_columns'),
                        'options' => array(
                            'one' => 1,
                            'two' => 2,
                        ),
                        'default' => 'two',
                    ),
                    array(
                        'id' => 'helpie_cp_article_text-typography',
                        'type' => 'typography',
                        'title' => Translations::get('al_typography'),
                        'output' => array('.helpie-single-page-module .helpiekb-box-list li a'),
                        'output_important' => true,
                    ),
                    array(
                        'id' => 'helpie_cp_article_border',
                        'type' => 'border',
                        'title' => Translations::get('al_border'),
                        'output' => array('.helpie-single-page-module .helpiekb-box-list li a'),
                        'output_important' => true,
                    ),
                    array(
                        'id' => 'helpie_cp_article_margin',
                        'type' => 'spacing',
                        'title' => Translations::get('al_margin'),
                        'output' => array('.helpie-single-page-module .helpiekb-box-list li a'),
                        'output_mode' => 'margin',
                        'output_important' => true,
                    ),
                    array(
                        'id' => 'helpie_cp_article_icon_color',
                        'type' => 'color',
                        'title' => Translations::get('al_icon_color'),
                        'output' => '.helpie-single-page-module .helpiekb-box-list li a::after',
                        'output_important' => true,
                    ),
                    array(
                        'type' => 'subheading',
                        'title' => Translations::get('ccl'),
                        'subtitle' => '',
                    ),
                    array(
                        'id' => 'helpie_cp_child_category_template',
                        'type' => 'select',
                        'chosen' => true,
                        'title' => Translations::get('ccl_style'),
                        'placeholder' => Translations::get('select_an_option'),
                        'options' => array(
                            'boxed' => Translations::get('boxed'),
                            /* boxed1 has issues  - no articles displayed for boxed1 */
                            // 'boxed1' => __('Boxed1', HELPIE_DOMAIN),
                            'modern' => Translations::get('modern'),
                        ),
                        'default' => 'boxed',
                        // 'dependency' => array('main_page_categories', '==', 'true'),
                    ),
                ),
            ));
        }

        public function get_capabilities_fields($prefix)
        {
            $cap_fields = $this->get_cap_fields();

            \CSF::createSection($prefix, array(
                // 'parent' => 'user_access',
                'title' => 'Can View',
                'icon' => 'fa fa-eye',
                'fields' => array(

                    array(
                        'id' => 'can_view',
                        'type' => 'fieldset',
                        'title' => Translations::get('who_can_view'),
                        'fields' => $cap_fields,
                    ),
                ),
            ));

            \CSF::createSection($prefix, array(
                // 'parent' => 'user_access',
                'title' => 'Can Edit',
                'icon' => 'fa fa-pencil',
                'fields' => array(

                    array(
                        'id' => 'can_edit',
                        'type' => 'fieldset',
                        'title' => Translations::get('who_can_edit'),
                        'fields' => $cap_fields,
                    ),
                ),
            ));

            \CSF::createSection($prefix, array(
                // 'parent' => 'user_access',
                'title' => 'Can Publish',
                'icon' => 'fa fa-book',
                'fields' => array(

                    array(
                        'id' => 'can_publish',
                        'type' => 'fieldset',
                        'title' => Translations::get('who_can_publish'),
                        'fields' => $cap_fields,
                    ),
                ),
            ));

            \CSF::createSection($prefix, array(
                // 'parent' => 'user_access',
                'title' => 'Can Approve',
                'icon' => 'fa fa-check',
                'fields' => array(

                    array(
                        'id' => 'can_approve',
                        'type' => 'fieldset',
                        'title' => Translations::get('who_can_approve'),
                        'fields' => $cap_fields,
                    ),
                ),
            ));
        }

        protected function get_all_usernames()
        {
            if (isset($this->usernames) && is_array($this->usernames)) {
                return $this->usernames;
            }

            $args = array(
                // 'number' => 10000,
                'offset' => 0,
                'fields' => array('ID', 'display_name'),
            );

            $users = \get_users($args);

            $this->usernames = array();

            foreach ($users as $user) {
                $this->usernames[$user->ID] = $user->display_name;
            }

            return $this->usernames;
        }

        public function get_cap_fields($scope = 'other', $cap = 'can_view')
        {
            $usernames = $this->get_all_usernames();

            // $roles = $this->get_editable_roles();

            $fields = array(
                array(
                    'id' => 'type',
                    'type' => 'select',
                    'title' => Translations::get('select_by'),
                    'placeholder' => Translations::get('select_an_option'),
                    'options' => array(
                        // 'default' => Translations::get('dc')__('Default from Parent / Global', HELPIE_DOMAIN),
                        'all' => Translations::get('all'),
                        'none' => Translations::get('no_one'),
                        'logged_in' => Translations::get('logged_in'),
                        'roles' => Translations::get('role'),
                        'user_id' => Translations::get('username'),
                    ),

                ),

                array(
                    'id' => 'rule',
                    'type' => 'select',
                    'dependency' => array('type', 'any', 'roles,user_id'),
                    'title' => Translations::get('rule'),
                    'placeholder' => Translations::get('select_an_option'),
                    'options' => array(
                        // 'default' => __('Default', HELPIE_DOMAIN),
                        'all_except' => Translations::get('everyone_except'),
                        'only' => Translations::get('only'),
                    ),
                    'default' => 'all',
                ),

                array(
                    'id' => 'usernames',
                    'type' => 'select',
                    'dependency' => array('type', '==', 'user_id'),
                    'chosen' => true,
                    'multiple' => true,
                    'title' => Translations::get('usernames'),
                    'placeholder' => Translations::get('select_an_option'),
                    'options' => $usernames,
                    // 'default' => 'option-2',
                ),
                array(
                    'id' => 'roles',
                    'type' => 'select',
                    'dependency' => array('type', '==', 'roles'),
                    'chosen' => true,
                    'multiple' => true,
                    'title' => Translations::get('roles'),
                    'placeholder' => Translations::get('select_an_option'),
                    'options' => 'get_dynamic_caps_roles',
                    'default' => 'option-2',
                ),

            );

            // Global should not have 'default'

            if ($scope == 'global') {
                if ($cap == 'can_view') {
                    $fields[0]['default'] = 'all';
                } else {
                    $fields[0]['default'] = 'none';
                }
            }

            if ($scope != 'global') {
                $fields[0]['options'] = array_merge(array('default' => Translations::get('default_parent_global')), $fields[0]['options']);
                $fields[0]['default'] = 'default';
            }

            return $fields;
        }

        public function can_view_posts()
        {

            $prefix = '_helpie_kb_post_options';

            \CSF::createMetabox($prefix, array(
                'title' => Translations::get('helpie_kb'),
                'post_type' => 'pauple_helpie',
                'show_restore' => true,
            ));

            $this->get_capabilities_fields($prefix);
        }

        public function can_view_topics()
        {
            // Field: fieldset

            $prefix = '_helpie_kb_options';

            // Create taxonomy options

            \CSF::createTaxonomyOptions($prefix, array(
                'taxonomy' => 'helpdesk_category',
            ));

            $this->get_capabilities_fields($prefix);
        }
    } // END CLASS
}
