<?php

namespace Helpie\Includes;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Helpie\Includes\Translations')) {
    class Translations
    {
        public static function get($stringName)
        {
            // Only set one time
            if (!isset($strings) || empty($strings)) {
                $site_strings = self::get_site_strings();
                $dashboard_strings = self::get_dashboard_strings();
                $strings = array_merge($site_strings, $dashboard_strings);
            }

            // Searched string is not exist display error for Developer insights
            if (!isset($strings[$stringName]) && empty($strings[$stringName])) {
                wp_die('"' . $stringName . '" translation string is not exist, Please add the given string in the translations.php file.');
            }

            return $strings[$stringName];
        }

        public static function get_site_strings()
        {
            $strings = [
                // category listing
                'more' => __('More', HELPIE_DOMAIN),
                'browse_all' => __('Browse All', HELPIE_DOMAIN),
                'articles_in_this_topic' => sprintf(__('%s in this topic', HELPIE_DOMAIN), __('Articles', HELPIE_DOMAIN)),
                'sub_topics_in_this_topic' => sprintf(__('%s in this topic', HELPIE_DOMAIN), __('Sub topics', HELPIE_DOMAIN)),
                'child_topics' => __('Child Topics', HELPIE_DOMAIN),

                // frontend editor
                'title_here' => __('Title here...', HELPIE_DOMAIN),
                'content_here' => __('Content here...', HELPIE_DOMAIN),
                'publish' => __('Publish', HELPIE_DOMAIN),
                'show_diff' => __('Show Diff', HELPIE_DOMAIN),
                'submit_for_review' => __('Submit For Review', HELPIE_DOMAIN),
                'revision' => __('Revision', HELPIE_DOMAIN),
                'revisions' => __("Revisions", HELPIE_DOMAIN),
                'remove_revision' => __('Remove Revision', HELPIE_DOMAIN),
                'categories' => __('Categories', HELPIE_DOMAIN),
                'tags' => __('Tags', HELPIE_DOMAIN),
                'press_enter' => __('Press Enter', HELPIE_DOMAIN),
                'add_category' => __("Add Category", HELPIE_DOMAIN),
                'select_category' => __('Select a Category', HELPIE_DOMAIN),
                'your_new_category' => __("Your New Category", HELPIE_DOMAIN),
                'featured_image' => __('Featured Image', HELPIE_DOMAIN),
                'no_revision_id' => __("No Revision ID", HELPIE_DOMAIN),
                'editor_parent_wiki_category' => __('-- Parent Wiki Category --', HELPIE_DOMAIN),
                'dont_have_access_to_edit' => __("You do not have access to edit this article", HELPIE_DOMAIN),

                // frontend editor popup
                'trash' => __('Trash', HELPIE_DOMAIN),
                'cancel' => __('Cancel', HELPIE_DOMAIN),

                // TODO: Use below translation in the popup.php
                'delete_popup_title' => __("Are you sure you want to %s ?"),

                // article author updated info
                'on' => __('on', HELPIE_DOMAIN),
                'read' => __('read', HELPIE_DOMAIN),
                'reads' => __('reads', HELPIE_DOMAIN),
                'anonymous' => __("Anonymous", HELPIE_DOMAIN),
                'last_updated_on' => sprintf(__('Last updated %s', HELPIE_DOMAIN), __('on', HELPIE_DOMAIN)),
                'last_updated_by' => sprintf(__('Last updated %s', HELPIE_DOMAIN), __('by', HELPIE_DOMAIN)),

                // Search
                'results_found_for' => __('results found for', HELPIE_DOMAIN),
                //stats
                'contributors' => __('Contributors', HELPIE_DOMAIN),
                //toc
                'no_headers_found' => __('No headers found ...', HELPIE_DOMAIN),
            ];

            return array_merge($strings, self::get_site_js_obj_strings());
        }

        public static function get_site_js_obj_strings()
        {
            // Single Cpt Name default to "Article"
            $settings = new \Helpie\Includes\Settings\Getters\Settings();
            $article = $settings->single_page->get_single_cpt_name();
            $search_settings = $settings->search->get_settings();

            /* Reference strings accessing from javascript */
            $reference_strings = [
                'noMatches' => $search_settings['empty_search_result_label'],
                'dirURL' => ABSPATH,
                'pluginURL' => HELPIE_PLUGIN_URL,
            ];

            /* String is passed to make javascript strings translatable */
            $strings = [
                'in' => __('in', HELPIE_DOMAIN),
                'viewed_revision_removed' => __('Viewed Revision Removed', HELPIE_DOMAIN),
                'set_featured_image' => sprintf(__('Set %s', HELPIE_DOMAIN), __("Featured Image", HELPIE_DOMAIN)),
                'choose_featured_image' => sprintf(__('Choose %s', HELPIE_DOMAIN), __("Featured Image", HELPIE_DOMAIN)),
                'new_article' => sprintf(__('New %s', HELPIE_DOMAIN), $article),
                'edit_article' => sprintf(__('Edit %s', HELPIE_DOMAIN), $article),
                'view_article' => sprintf(__('View %s', HELPIE_DOMAIN), $article),
                'article_published' => sprintf(__('%s Published', HELPIE_DOMAIN), $article),
                'article_submitted' => sprintf(__('%s Submitted', HELPIE_DOMAIN), $article),
                'article_deleted' => sprintf(__('%s Deleted', HELPIE_DOMAIN), $article),
                'article_trashed' => sprintf(__('%s Trashed', HELPIE_DOMAIN), $article),
                'visit_main_page' => sprintf(__('Visit %s', HELPIE_DOMAIN), __('Main Page', HELPIE_DOMAIN)),
            ];

            return array_merge($strings, $reference_strings);
        }

        public static function get_dashboard_strings()
        {
            $dashboard_strings = [
                'helpie_settings' => __('Helpie Settings', HELPIE_DOMAIN),
            ];

            $strings = [];

            $common_strings = self::get_widgets_notices_strings();

            $strings['main_page'] = self::get_mp_strings();

            $strings['category_page'] = self::get_cp_strings();

            $strings['single_article_page'] = self::get_sp_strings();

            $strings['search'] = self::get_search_strings();

            $strings['frontend_editor'] = self::get_frontend_editor_strings();

            $strings['design'] = self::get_design_strings();

            $strings['password_settings'] = self::get_pp_strings();

            $strings['dynamic_capabilities'] = self::get_dynamic_caps_strings();

            $strings['table_of_contents'] = self::get_toc_strings();

            $strings['autolinking'] = self::get_autolinking_strings();

            $strings['components'] = self::get_components_strings();

            $strings['demo_import'] = self::get_demo_import_strings();

            $strings['backup_restore'] = self::get_backup_restore_strings();

            $strings['performance'] = self::get_performance_strings();

            $strings['insights'] = self::get_insights_strings();

            $strings['cpt'] = self::get_cpt_strings();

            $strings['envato'] = self::get_envato_codecanyon_strings();

            $strings = array_merge(
                $dashboard_strings,
                $common_strings,
                $strings['main_page'],
                $strings['category_page'],
                $strings['single_article_page'],
                $strings['search'],
                $strings['frontend_editor'],
                $strings['design'],
                $strings['password_settings'],
                $strings['dynamic_capabilities'],
                $strings['table_of_contents'],
                $strings['autolinking'],
                $strings['components'],
                $strings['demo_import'],
                $strings['backup_restore'],
                $strings['performance'],
                $strings['insights'],
                $strings['cpt'],
                $strings['envato']
            );

            return $strings;
        }

        protected static function get_widgets_notices_strings()
        {
            $single = __('Single', HELPIE_DOMAIN);
            $single_item = __('Single Item', HELPIE_DOMAIN);
            return [
                // hero area
                'search' => __('Search', HELPIE_DOMAIN),
                'sub_title' => __('Sub Title', HELPIE_DOMAIN),
                'single_item' => $single_item,
                'single_icon' => sprintf(__('Single Icon', HELPIE_DOMAIN), $single),
                'single_item_icon' => sprintf(__('%s Icon', HELPIE_DOMAIN), $single_item),
                'single_item_title' => sprintf(__('%s Title', HELPIE_DOMAIN), $single_item),
                'single_item_label' => sprintf(__('%s Label', HELPIE_DOMAIN), $single_item),
                'single_item_count' => sprintf(__('%s Count', HELPIE_DOMAIN), $single_item),
                'single_item_content' => sprintf(__('%s Content', HELPIE_DOMAIN), $single_item),
                'display_sub_title' => sprintf(__('Show %s', HELPIE_DOMAIN), __('Sub Title', HELPIE_DOMAIN)),
                // article listing
                'title_icon' => sprintf(__('%s Icon', HELPIE_DOMAIN), __('Title', HELPIE_DOMAIN)),
                // breadcrumbs
                'label' => __('Label', HELPIE_DOMAIN),
                'breadcrumbs_container' => __('Breadcrumbs Container', HELPIE_DOMAIN),
                'link' => __('Link', HELPIE_DOMAIN),
                'voting_icon' => sprintf(__('%s Icon', HELPIE_DOMAIN), __('Voting', HELPIE_DOMAIN)),
                // category listing
                'display_widget_title' => __('Show Widget Title', HELPIE_DOMAIN),
                'article_count' => sprintf(__('%s Count', HELPIE_DOMAIN), __('Article', HELPIE_DOMAIN)),
                'custom' => __('Custom', HELPIE_DOMAIN),
                'type' => __('Type', HELPIE_DOMAIN),
                'num_of_articles' => __('Number of Articles', HELPIE_DOMAIN),
                // voting
                'button' => __('Button', HELPIE_DOMAIN),
                'button_icon' => sprintf(__('%s Icon', HELPIE_DOMAIN), __('Button', HELPIE_DOMAIN)),
                // stats
                'stats_collection' => __('Stats Collection', HELPIE_DOMAIN),
                'stats_title' => __('Stats Title', HELPIE_DOMAIN),
                'display_title' => __('Show Title', HELPIE_DOMAIN),
                // toc
                'article_limit' => sprintf(__('%s Limit', HELPIE_DOMAIN), __('Article', HELPIE_DOMAIN)),
                'child_category_limit' => sprintf(__('%s Limit', HELPIE_DOMAIN), __('Child Category', HELPIE_DOMAIN)),
                'toggle_category_children' => __('Toggle Category Children', HELPIE_DOMAIN),
                'show_auto_toc' => __('Show Auto TOC', HELPIE_DOMAIN),
                // Admin columns
                'order_in_cat' => __('Order in Category', HELPIE_DOMAIN),
                // Notices
                'welcome_to_helpie_kb' => __('Welcome to Helpie KB Wiki', HELPIE_DOMAIN),
                'go_to_setup_page' => __('Go to Setup Page', HELPIE_DOMAIN),
                'skip' => __('Skip', HELPIE_DOMAIN),
                'dismiss_permanently' => __('Dismiss Permanently', HELPIE_DOMAIN),
                'dismiss_this_notice' => __('Dismiss this notice.', HELPIE_DOMAIN),
                'onboard_page_notice_content' => __('Setup your first content and choose your favourite demo to import !!!', HELPIE_DOMAIN),
                'search_page_notice_content' => __('Helpdesk Search Page was deleted. Do you want to', HELPIE_DOMAIN),
                'frontend_page_notice_content' => __('Frontend Editor Page was deleted. Do you want to', HELPIE_DOMAIN),
                'the_page' => __('the page', HELPIE_DOMAIN),
                'restore' => __('restore', HELPIE_DOMAIN),
            ];
        }

        protected static function get_mp_strings()
        {
            $extras = new \Helpie\Includes\Settings\Partials\Extras();

            $main_page = __('Main Page', HELPIE_DOMAIN);
            $template = __('Template', HELPIE_DOMAIN);

            return [
                'main_page' => $main_page,
                'main_page_settings' => sprintf(__('%s Settings', HELPIE_DOMAIN), $main_page),
                // 'visit_main_page' => sprintf(__('Visit %s', HELPIE_DOMAIN), $main_page),
                'template' => $template,
                //  "%1$s %2$s"
                'mp_template_help' => sprintf(__('%1$s Type for your Knowledge Base\'s %2$s. Both Side Sidebars, Left / Right Sidebar, Full-Width ( without wrapper ), Boxed-Width.', HELPIE_DOMAIN), $template, $main_page),

                'sidebar_widget_help' => __('You can add widgets to your sidebar from Appearance -> Widgets', HELPIE_DOMAIN),
                'sidebar1' => sprintf(__('Sidebar %d', HELPIE_DOMAIN), 1),
                'sidebar2' => sprintf(__('Sidebar %d', HELPIE_DOMAIN), 2),
                'select_an_option' => __('Select an option', HELPIE_DOMAIN),

                'mp_customize' => sprintf(__('There are 4 ways to customize the %s', HELPIE_DOMAIN), $main_page)
                . '<br><br>1) ' . __('Use the settings below', HELPIE_DOMAIN)
                . '<br> 2) ' . sprintf(__('Change &ldquo;%s&rdquo; and choose a page, then apply the shortcode mentioned', HELPIE_DOMAIN), __('Where do you want your main page?', HELPIE_DOMAIN))
                . '<br>3) ' . __('Change the main page and use Elementor page builder', HELPIE_DOMAIN)
                . '<br> 4) <a href="http://helpiewp.com/knowledge-base-elementor-template-builder-integration/">' . __('Use Elementor Pro Theme Builder', HELPIE_DOMAIN) . '</a>'
                . '( ' . __('Need\'s to enable below CPT Archive option', HELPIE_DOMAIN) . ' )',

                'mp_location' => __('Where do you want your main page?', HELPIE_DOMAIN),
                'mp_location_cpt' => __('CPT Archive', HELPIE_DOMAIN),
                'mp_location_page' => __('Select a Page', HELPIE_DOMAIN),
                'mp_select_page' => __('Select Page to be KB Main Page', HELPIE_DOMAIN),

                'mp_select_page_help' => sprintf(__('%s Add this shortcode to the selected page', HELPIE_DOMAIN), '<b>[pauple_helpie_main_page_shortcode]</b> ->'),

                'mp_slug' => __('Main Page Slug', HELPIE_DOMAIN),

                'mp_link' => '<div class="button-container">'
                . '<span><b>' . __('Where is my main page?', HELPIE_DOMAIN) . '</b></span>'
                . '<br>'
                . $extras->get_main_page_url() . '<span>' . __('Save and Refresh Page if you changed it', HELPIE_DOMAIN) . '.</span></div>',

                'hero_section' => __('Hero Section', HELPIE_DOMAIN),
                'hero_section_subtitle' => __('Style settings for Hero Section are in Design sub-menu', HELPIE_DOMAIN),
                'hero_section_control' => __('Hero Section Control', HELPIE_DOMAIN),
                'mp_controls_help' => __('Control order and visibility of these components in Main Page', HELPIE_DOMAIN),
                'kb_title' => __('Knowledge Base Main Title', HELPIE_DOMAIN),
                'kb_title_help' => __('Displayed in Hero section and also in Breadcrumbs.', HELPIE_DOMAIN),
                'kb_subtitle' => __('Knowledge Base Subtitle', HELPIE_DOMAIN),
                'kb_hero_titles_help' => __('Displayed in Hero section.', HELPIE_DOMAIN),
                'search_display' => __('Search Display', HELPIE_DOMAIN),
                'mp_meta_data' => __('Meta Data', HELPIE_DOMAIN),
                'mp_meta_data_archive' => sprintf(__('This works only when you set CPT Archive for %s', HELPIE_DOMAIN), __('Where do you want your main page?', HELPIE_DOMAIN)),
                'mp_meta_title' => __('Main Page Meta Title', HELPIE_DOMAIN),
                'mp_meta_title_help' => __('Keep your meta title between 60 and 64 characters', HELPIE_DOMAIN),
                'mp_meta_desc' => __('Main Page Meta Description', HELPIE_DOMAIN),
                'mp_meta_desc_help' => __('Keep your meta description between 150 and 154 characters', HELPIE_DOMAIN),
                'mp_components' => sprintf(__('%s Settings', HELPIE_DOMAIN), $main_page . ' ' . __('Components', HELPIE_DOMAIN)),
                'mp_components_control' => __('Components Control', HELPIE_DOMAIN),
                'mp_controls_help' => __('Control order and visibility of these components in Main Page', HELPIE_DOMAIN),
                'mp_show_stats' => __('Show Stats', HELPIE_DOMAIN),
                'mp_categories' => $main_page . ' - ' . __('Categories', HELPIE_DOMAIN),
                'article_listing' => __('Article Listing', HELPIE_DOMAIN),
                'mp_article_listing' => sprintf(__('Show %s', HELPIE_DOMAIN), __('Article Listing', HELPIE_DOMAIN)),
                'title' => __('Title', HELPIE_DOMAIN),
                'sort_by' => __('Sort By', HELPIE_DOMAIN),
                'alphabetical' => __('Alphabetical', HELPIE_DOMAIN),
                'recent' => __('Recent', HELPIE_DOMAIN),
                'recently_updated' => __('Recently Updated', HELPIE_DOMAIN),
                'popular' => __('Popular', HELPIE_DOMAIN),
                'topics' => __('Topics', HELPIE_DOMAIN),
                'style' => __('Style', HELPIE_DOMAIN),
                'list' => __('List', HELPIE_DOMAIN),
                'card' => __('Card', HELPIE_DOMAIN),
                'num_of_cols' => __('Number Of Columns', HELPIE_DOMAIN),
                'show_image' => __('Show Image', HELPIE_DOMAIN),
                'show_extra_info' => __('Show Extra Info', HELPIE_DOMAIN),
                'true' => __('True', HELPIE_DOMAIN),
                'false' => __('False', HELPIE_DOMAIN),
                'limit' => __('Limit', HELPIE_DOMAIN),
                'category_listing' => __('Category Listing', HELPIE_DOMAIN),
                'mp_category_listing_style' => __('Main Page Categories Listing Style', HELPIE_DOMAIN),
                'boxed' => __('Boxed', HELPIE_DOMAIN),
                'boxed1' => __('Boxed1', HELPIE_DOMAIN),
                'modern' => __('Modern', HELPIE_DOMAIN),
                'show_description' => __('Show Description', HELPIE_DOMAIN),
                'image_or_icon' => __('Image or Icon', HELPIE_DOMAIN),
                'image' => __('Image', HELPIE_DOMAIN),
                'icon' => __('Icon', HELPIE_DOMAIN),
                'mp_category_listing_graphic_help' => __('Default icon color is set from Style -> Primary Brand color', HELPIE_DOMAIN),
                'children_type' => __('Children Type', HELPIE_DOMAIN),
                'dont_show_children' => __('Don\'t show Children', HELPIE_DOMAIN),
                'articles' => __('Articles', HELPIE_DOMAIN),
                'sub_categories' => __('Sub Categories', HELPIE_DOMAIN),
                'mp_no_cat_articles' => __('Number of articles under each category', HELPIE_DOMAIN),
                'mp_cats' => $main_page . ' ' . __('Categories', HELPIE_DOMAIN),
                'enabled' => __('Enabled', HELPIE_DOMAIN),
                'disabled' => __('Disabled', HELPIE_DOMAIN),
            ];
        }

        protected static function get_cp_strings()
        {
            $article_list = sprintf(__('%s List', HELPIE_DOMAIN), __('Article', HELPIE_DOMAIN));
            $child_category = __('Child Category', HELPIE_DOMAIN);
            $child_category_list = sprintf(__('%s List', HELPIE_DOMAIN), $child_category);
            $child_category_listing = sprintf(__('%s Listing', HELPIE_DOMAIN), $child_category);

            return [
                'category_page' => __('Category Page', HELPIE_DOMAIN),
                'category_page_settings' => sprintf(__('%s Settings', HELPIE_DOMAIN), __('Category Page', HELPIE_DOMAIN)),
                'cp_slug' => sprintf(__('%s Slug', HELPIE_DOMAIN), __('Category Page', HELPIE_DOMAIN)),
                'cp_title_color' => sprintf(__('%s - Title Color', HELPIE_DOMAIN), __('Category Page', HELPIE_DOMAIN)),
                'al_style' => sprintf(__('%s - Style', HELPIE_DOMAIN), $article_list),
                'al_no_of_columns' => $article_list . ' - ' . __('Number Of Columns', HELPIE_DOMAIN),
                'minimal' => __('Minimal', HELPIE_DOMAIN),
                'al_typography' => sprintf(__('%s - Typography', HELPIE_DOMAIN), $article_list),
                'al_border' => sprintf(__('%s - Border', HELPIE_DOMAIN), $article_list),
                'al_margin' => sprintf(__('%s - Margin', HELPIE_DOMAIN), $article_list),
                'al_icon_color' => sprintf(__('%s - Icon Color', HELPIE_DOMAIN), $article_list),
                'ccl' => $child_category_listing,
                'ccl_style' => sprintf(__('%s - Style', HELPIE_DOMAIN), $child_category_list),
            ];
        }

        protected static function get_sp_strings()
        {
            $show = __('Show', HELPIE_DOMAIN);
            $single = __('Single', HELPIE_DOMAIN);
            $single_post = __('Single Post', HELPIE_DOMAIN);
            $voting = __('Voting', HELPIE_DOMAIN);

            return [
                'single_page' => __('Single Article Page', HELPIE_DOMAIN),
                'sp_template' => sprintf(__('%s - Template', HELPIE_DOMAIN), $single),
                'sp_template_help' => __('These settings are only for Helpie KB post type.', HELPIE_DOMAIN),

                'sp_template_source_help' => '<b>' . sprintf(__('%s - Template Source', HELPIE_DOMAIN), $single) . '</b> ' . __('Use the Theme Template to get', HELPIE_DOMAIN) . ' : </br>'
                . '1) ' . __('Your Theme Template', HELPIE_DOMAIN)
                . '</br>2) ' . __('Elementor Theme Builder Template (not for page builder)', HELPIE_DOMAIN),

                'sp_template_source' => sprintf(__('%s - Template Source', HELPIE_DOMAIN), $single),
                'sp_helpie_template' => __('Helpie Plugin Single Default Template', HELPIE_DOMAIN),
                'sp_theme_template' => __('Theme Single Template', HELPIE_DOMAIN),
                'sidebar_widget_toc_help' => __("You can add HelpieKB's Table of Contents to your sidebar widgets", HELPIE_DOMAIN),
                'sp_labelling' => sprintf(__('%s - Labelling', HELPIE_DOMAIN), $single),
                'sp_cpt_label' => sprintf(__('%s Label', HELPIE_DOMAIN), $single_post),
                'sp_cpt_label_help' => __("Changes the Single post Wiki Label 'Article' into your Custom Label.", HELPIE_DOMAIN),
                'sp_cpt_label_plural' => sprintf(__('%s Label Plural', HELPIE_DOMAIN), $single_post),
                'sp_display_author_name' => sprintf(__('%s Author Name', HELPIE_DOMAIN), $show),
                'sp_display_updatedon' => sprintf(__('%s Last Updated On', HELPIE_DOMAIN), $show),
                'sp_display_pageviews' => sprintf(__('%s Number of Reads', HELPIE_DOMAIN), $show),
                'voting' => $voting,
                'voting_system' => sprintf(__('%s System', HELPIE_DOMAIN), $voting),
                'classic' => __('Classic', HELPIE_DOMAIN),
                'emotion' => __('Emotion', HELPIE_DOMAIN),
                'none' => __('None', HELPIE_DOMAIN),
                'voting_label' => sprintf(__('%s Label', HELPIE_DOMAIN), $voting),
                'voting_access' => __('Allow non-logged in users to vote', HELPIE_DOMAIN),
                'comments' => __('Comments', HELPIE_DOMAIN),
                'show_comments' => sprintf(__('%s Comments', HELPIE_DOMAIN), $show),
                'sp_title_color' => sprintf(__('%s - Title Color', HELPIE_DOMAIN), $single_post),
            ];
        }

        protected static function get_search_strings()
        {
            $search = __('Search', HELPIE_DOMAIN);
            $search_border = __('Search Border', HELPIE_DOMAIN);
            $desc_length = __('Description Length', HELPIE_DOMAIN);

            return [
                'search_settings' => sprintf(__('%s Settings', HELPIE_DOMAIN), $search),
                'search_select_page' => __('Select page to be a search page', HELPIE_DOMAIN),
                'search_select_page_help' => sprintf(__('%s Add this shortcode to the selected page', HELPIE_DOMAIN), '<b>[pauple_helpie_search_results_page] </b> ->'),
                'search_placeholder_txt' => __('Search placeholder text', HELPIE_DOMAIN),
                'search_no_query_txt' => __('No Query Text', HELPIE_DOMAIN),
                'search_empty_result_label' => __('Empty search results label', HELPIE_DOMAIN),
                'search_page_featured_image_show' => __('Show Featured Image', HELPIE_DOMAIN),
                'search_page_meta_data_show' => __('Show Metadata', HELPIE_DOMAIN),
                'search_page_description_show' => __('Show Result Description', HELPIE_DOMAIN),
                'search_page_tags_show' => __('Show Tags', HELPIE_DOMAIN),
                'search_page_desc_length' => $desc_length,
                'search_page_desc_length_help' => sprintf(__('%s Set value to "-1" to get max value', HELPIE_DOMAIN), '<b>' . $desc_length . ':</b> '),
                'search_style_help' => __('Search Results', HELPIE_DOMAIN),
                'search_header_typography' => sprintf(__('%s Typography', HELPIE_DOMAIN), __('Header', HELPIE_DOMAIN)),
                'search_title_typography' => sprintf(__('%s Typography', HELPIE_DOMAIN), __('Title', HELPIE_DOMAIN)),
                'search_category_typography' => sprintf(__('%s Typography', HELPIE_DOMAIN), __('Category', HELPIE_DOMAIN)),
                'search_meta_typography' => sprintf(__('%s Typography', HELPIE_DOMAIN), __('Metadata', HELPIE_DOMAIN)),
                'search_text_typography' => sprintf(__('%s Typography', HELPIE_DOMAIN), __('Search Text', HELPIE_DOMAIN)),
                'border' => __('Border', HELPIE_DOMAIN),
                'padding' => __('Padding', HELPIE_DOMAIN),
                'margin' => __('Margin', HELPIE_DOMAIN),
                'search_box' => sprintf(__('Search Box', HELPIE_DOMAIN), $search),
                'search_border_display' => sprintf(__('%s Display', HELPIE_DOMAIN), $search_border),
                'search_border_color' => sprintf(__('%s Color', HELPIE_DOMAIN), $search_border),
                'search_border_style' => sprintf(__('%s Style', HELPIE_DOMAIN), $search_border),
                'rounded' => __('Rounded', HELPIE_DOMAIN),
                'semi_rounded' => __('Semi Rounded', HELPIE_DOMAIN),
                'squared' => __('Squared', HELPIE_DOMAIN),
                'posts_per_page' => __('Posts Per Page', HELPIE_DOMAIN),
                'range' => __('Range', HELPIE_DOMAIN),
                'search_results_container' => __('Search Results Container', HELPIE_DOMAIN),
                'search_single_result' => __('Single Result', HELPIE_DOMAIN),
                'header' => __('Header', HELPIE_DOMAIN),
                'post_title' => __('Post Title', HELPIE_DOMAIN),
                'category_name' => __('Category Name', HELPIE_DOMAIN),
                'meta_icons' => __('Meta Icons', HELPIE_DOMAIN),
                'meta_values' => __('Meta Values', HELPIE_DOMAIN),
                'excerpt' => __('Excerpt', HELPIE_DOMAIN),
                'voting_container' => __('Voting Container', HELPIE_DOMAIN),
                'placeholder' => __('Placeholder', HELPIE_DOMAIN),
            ];
        }

        protected static function get_frontend_editor_strings()
        {
            return [
                'frontend_editor' => __('Frontend Editor', HELPIE_DOMAIN),
                'enable_editor' => __('Enable Frontend Editing', HELPIE_DOMAIN),
                'num_of_revision' => __('Number of Revisions', HELPIE_DOMAIN),
                'editor_type' => __('Editor Type', HELPIE_DOMAIN),
                'inline_editor' => __('Inline Editor', HELPIE_DOMAIN),
                'tinymce_editor' => __('WordPress Tinymce Editor', HELPIE_DOMAIN),
            ];

        }

        protected static function get_design_strings()
        {
            $hero_area = __('Hero Area', HELPIE_DOMAIN);
            $overall_margin_top = __('Overall margin-top', HELPIE_DOMAIN);
            return [
                'design' => __('Design', HELPIE_DOMAIN),
                'typography' => __('Typography', HELPIE_DOMAIN),
                'layout' => __('Layout', HELPIE_DOMAIN),
                'wrapper_width' => __('Wrapper Width', HELPIE_DOMAIN),
                'kb_content_area' => __('KB Content Area', HELPIE_DOMAIN),
                'margin_top_desktop' => sprintf(__('%s - Desktop', HELPIE_DOMAIN), $overall_margin_top),
                'margin_top_tablet' => sprintf(__('%s - Tablet', HELPIE_DOMAIN), $overall_margin_top),
                'margin_top_mobile' => sprintf(__('%s - Mobile', HELPIE_DOMAIN), $overall_margin_top),
                'heror_area_background_type' => sprintf(__('%s background style', HELPIE_DOMAIN), $hero_area),
                'primary_color' => __('Use Primary Color', HELPIE_DOMAIN),
                'color_gradient' => __('Color Gradient', HELPIE_DOMAIN),
                'background_image' => __('Background Image', HELPIE_DOMAIN),
                'gradient_plus_background_image' => __('Gradient + Illustration', HELPIE_DOMAIN),
                'primary_brand_color' => __('Primary Brand Color', HELPIE_DOMAIN),
                'heror_area_image' => sprintf(__('%s Image', HELPIE_DOMAIN), $hero_area),
                'heror_area_illustration' => sprintf(__('%s illustration', HELPIE_DOMAIN), $hero_area),
                'heror_area_gradient1' => sprintf(__('%s Gradient Color %d', HELPIE_DOMAIN), $hero_area, 1),
                'heror_area_gradient2' => sprintf(__('%s Gradient Color %d', HELPIE_DOMAIN), $hero_area, 2),
                'heror_area_title_color' => sprintf(__('%s - Title Color', HELPIE_DOMAIN), $hero_area),
                'heror_area_text_color' => sprintf(__('%s - Text color', HELPIE_DOMAIN), $hero_area),
                'heror_area_section_padding' => __('Main Hero Section - Padding', HELPIE_DOMAIN),
                'kb_typography' => sprintf(__('%s - Typography', HELPIE_DOMAIN), __('Knowledgebase', HELPIE_DOMAIN)),
                'h1_typography' => sprintf(__('h%d - Typography', HELPIE_DOMAIN), 1),
                'h2_typography' => sprintf(__('h%d - Typography', HELPIE_DOMAIN), 2),
                'h3_typography' => sprintf(__('h%d - Typography', HELPIE_DOMAIN), 3),
                'h4_typography' => sprintf(__('h%d - Typography', HELPIE_DOMAIN), 4),
                'h5_typography' => sprintf(__('h%d - Typography', HELPIE_DOMAIN), 5),
                'h6_typography' => sprintf(__('h%d - Typography', HELPIE_DOMAIN), 6),
                'ptag_typography' => sprintf(__('%s - Typography', HELPIE_DOMAIN), __('p tag', HELPIE_DOMAIN)),
            ];
        }

        protected static function get_pp_strings()
        {
            $password = __('Password', HELPIE_DOMAIN);
            return [
                'password_settings' => sprintf(__('%s Settings', HELPIE_DOMAIN), $password),
                'pp_not_required' => __('Passwords are not required for admin', HELPIE_DOMAIN),
                'pp_plural' => __('Passwords', HELPIE_DOMAIN),
                'pp_singular' => $password,
                'category' => __('Category', HELPIE_DOMAIN),
                'pp_remember_days' => __('Remember password for this many days', HELPIE_DOMAIN),
            ];
        }

        protected static function get_dynamic_caps_strings()
        {
            return [
                'dynamic_capabilities' => __('Dynamic Capabilities', HELPIE_DOMAIN),
                'dc_non_admin_restriction' => __('These restrictions do not apply for admin', HELPIE_DOMAIN),

                'dc_non_admin_restriction_help' => __('You can control these conditions at Global, Topic and Article levels', HELPIE_DOMAIN) . '. <a href="http://helpiewp.com/dynamic-capabilities/">' . __('Learn More', HELPIE_DOMAIN) . ' >></a>',

                'can_view' => __('Can View', HELPIE_DOMAIN),
                'who_can_view' => __('Who Can View', HELPIE_DOMAIN),
                'can_edit' => __('Can Edit', HELPIE_DOMAIN),
                'who_can_edit' => __('Who Can Edit', HELPIE_DOMAIN),
                'can_publish' => __('Can Publish', HELPIE_DOMAIN),
                'who_can_publish' => __('Who Can Publish', HELPIE_DOMAIN),
                'can_approve' => __('Can Approve', HELPIE_DOMAIN),
                'who_can_approve' => __('Who Can Approve', HELPIE_DOMAIN),
                'select_by' => __('Select By', HELPIE_DOMAIN),
                'all' => __('All', HELPIE_DOMAIN),
                'no_one' => __('No One', HELPIE_DOMAIN),
                'logged_in' => __('Logged in', HELPIE_DOMAIN),
                'role' => __('Role', HELPIE_DOMAIN),
                'roles' => __('Roles', HELPIE_DOMAIN),
                'username' => __('User name', HELPIE_DOMAIN),
                'usernames' => __('Usernames', HELPIE_DOMAIN),
                'rule' => __('Rule', HELPIE_DOMAIN),
                'everyone_except' => __('Everyone Except', HELPIE_DOMAIN),
                'only' => __('Only', HELPIE_DOMAIN),
                'default_parent_global' => __('Default from Parent / Global', HELPIE_DOMAIN),
                'helpie_kb' => __('Helpie KB', HELPIE_DOMAIN),
            ];
        }

        protected static function get_toc_strings()
        {
            return [
                'toc' => __('Table of Contents', HELPIE_DOMAIN),
                'toc_title' => __('Table of Contents Title', HELPIE_DOMAIN),
                'toc_fixed_sidebar' => __('Fixed Sidebar Position', HELPIE_DOMAIN),
                'toc_sidebar_type' => __('Sidebar Type', HELPIE_DOMAIN),
                'toc_full_nav' => __('Full Navigation', HELPIE_DOMAIN),
                'toc_scroll_only' => __('In-Page Navigation only', HELPIE_DOMAIN),

                'toc_sidebar_type_help' => __('If you Select In-Page-Only other than single page default to Full Navigation', HELPIE_DOMAIN),

                'toc_full_nav_settings' => __('Full Navigation Settings', HELPIE_DOMAIN),
                'toc_full_nav_settings_sub' => __('Full Navigation shows categories, articles, etc', HELPIE_DOMAIN),
                'toc_display_in_page_nav' => __('Show In-Page Navigation in Table Of Contents', HELPIE_DOMAIN),
                'toc_display_in_page_nav_help' => __('Showing In-Page-Nav Within a Current Post in Single Page Only', HELPIE_DOMAIN),
                'toc_sidebar_categories' => __('Sidebar Categories', HELPIE_DOMAIN),

                'toc_sidebar_categories_help' => __('If you Select Main Page options Sidebar will display Categories of Main Page', HELPIE_DOMAIN),

                'toc_category_anchor_link' => __('Category anchor link', HELPIE_DOMAIN),
                'toc_display_num_of_child_category' => __('Number of child category to show', HELPIE_DOMAIN),
                'toc_display_num_of_articles' => __('Number of articles to show under each category', HELPIE_DOMAIN),
                'toc_display_articles' => __('Show Articles in TOC', HELPIE_DOMAIN),

                'performance_warning' => __("Performance Warning: Disable option Table of Contents -> Show Articles in TOC option to improve loading speed (if you have any issues)).", HELPIE_DOMAIN),

                'toc_in_page_nav_settings' => __('In-Page Navigation Settings', HELPIE_DOMAIN),
                'toc_in_page_nav_settings_sub' => __('In-Page Navigation is built from h1-h6 tags found in the article', HELPIE_DOMAIN),

                'toc_exclude_headings' => __('Exclude Headings', HELPIE_DOMAIN),
                'toc_exclude_headings_help' => __('Lower level headings without parents will be treated as root entries', HELPIE_DOMAIN),

                'toc_in_page_nav_title' => __('In-Page Navigation Title', HELPIE_DOMAIN),
                'toc_numeric_bullet' => __('Numeric Bullet', HELPIE_DOMAIN),
                'toc_section_page_url' => __('Section Page URL', HELPIE_DOMAIN),
                'toc_section_page_url_text' => __('Section Page URL Text', HELPIE_DOMAIN),
                'toc_back_to_top_link' => __('Back to top link', HELPIE_DOMAIN),
                'toc_back_to_top_text' => __('Back to top Text', HELPIE_DOMAIN),
                'toc_scroll_back_to_site_top' => __('Scroll Back to Site Top', HELPIE_DOMAIN),
                'toc_smooth_scroll' => __('Smooth Scroll', HELPIE_DOMAIN),
            ];
        }

        protected static function get_autolinking_strings()
        {
            return [
                'autolinking' => __('Autolinking', HELPIE_DOMAIN),
                'enable_autolinking' => __('Enable Auto Linking', HELPIE_DOMAIN),
                'enable_autolinking_help' => __('Enable Wiki Auto Linking', HELPIE_DOMAIN),
            ];
        }

        protected static function get_components_strings()
        {
            return [
                'components' => __('Components', HELPIE_DOMAIN),
                'breadcrumbs' => __('Breadcrumbs', HELPIE_DOMAIN),
                'article_order' => __('Article Order', HELPIE_DOMAIN),
                'menu_order' => __('Menu Order', HELPIE_DOMAIN),
                'asc_post_date' => __('Asc Post Date', HELPIE_DOMAIN),
                'desc_post_date' => __('Desc Post Date', HELPIE_DOMAIN),
            ];
        }

        protected static function get_demo_import_strings()
        {
            $onboarding = [
                'onboarding' => __('Onboarding', HELPIE_DOMAIN),
                'importing' => __('Importing', HELPIE_DOMAIN),
                'page_setup' => __('Page Setup', HELPIE_DOMAIN),
                'page_setup_sub' => __('Manage basic main page settings', HELPIE_DOMAIN),
                'page_setup_step' => __('Basic main page settings', HELPIE_DOMAIN),
                'demo_setup' => __('Demo Content & Dummy Content', HELPIE_DOMAIN),
                'demo_setup_sub' => __('Content to get started', HELPIE_DOMAIN),
                'demo_setup_step_content' => __('Demo Content', HELPIE_DOMAIN),
                'finish' => __('Finish', HELPIE_DOMAIN),
                'finish_sub' => __('Successfully, Setup completed', HELPIE_DOMAIN),
                'demo_setup_content' => __('Choose Demo Style', HELPIE_DOMAIN),
                'select_categories_to_add' => __('Select Categories to add', HELPIE_DOMAIN),
                'page_setup_content' => __('What do you want to create', HELPIE_DOMAIN),
                'knowledge_base' => __('Knowledge Base', HELPIE_DOMAIN),
                'wiki' => __('Wiki', HELPIE_DOMAIN),
                'documentation' => __('Documentation', HELPIE_DOMAIN),
                'name_of_ur_kb' => __('Name of your Knowledge Base', HELPIE_DOMAIN),
                'page_slug_label' => __('Slug ( The last part of the URL you see in the browser )', HELPIE_DOMAIN),
                'best_reproduction_demo_info' => __('For the best reproduction of the demos, use:', HELPIE_DOMAIN),
                'elementor_page_builder' => __('Elementor Page Builder', HELPIE_DOMAIN),
                'astra_theme' => __('Astra Theme', HELPIE_DOMAIN),
                'helpie_faq_plugin' => __('Helpie FAQ Plugin', HELPIE_DOMAIN),
                'previous' => __('Previous', HELPIE_DOMAIN),
                'next' => __('Next', HELPIE_DOMAIN),
            ];

            $demo_import = [
                'demo' => __('Demo', HELPIE_DOMAIN),
                'demo_import' => __('Demo Import', HELPIE_DOMAIN),
                'delete_notice' => __('Selected content deleted!', HELPIE_DOMAIN),
                'couldnt_delete_notice' => __('Couldn\'t able to delete content!', HELPIE_DOMAIN),
                'select_all' => __('Select All', HELPIE_DOMAIN),
                'delete' => __('Delete', HELPIE_DOMAIN),
                'demo_sub' => __('Manage your imported demo entries', HELPIE_DOMAIN),
                'all_done_have_fun' => __('All done. Have fun!', HELPIE_DOMAIN),
                'invalid_file_type' => __('Invalid file type', HELPIE_DOMAIN),
                'remote_server_not_respond' => __('Remote server did not respond', HELPIE_DOMAIN),
                'remote_file_incorrect' => __('Remote file is incorrect size', HELPIE_DOMAIN),
                'zero_size_file_downloaded' => __('Zero size file downloaded', HELPIE_DOMAIN),

                // TODO: Use below translation in the importer.php
                'failed_to_import_author' => __('Failed to import author %s. Their posts will be attributed to the current user.', HELPIE_DOMAIN),
                'failed_to_create_new_user' => __('Failed to create new user for %s. Their posts will be attributed to the current user.', HELPIE_DOMAIN),
                'failed_to_category' => __('Failed to import category %s', HELPIE_DOMAIN),
                'failed_to_import_post_tag' => __('Failed to import post tag %s', HELPIE_DOMAIN),
                'term_already_exist' => __('Term %s already exists.', HELPIE_DOMAIN),
                'failed_to_import_tax_term' => __('Failed to import %s %s', HELPIE_DOMAIN),
                'failed_to_import_post_cpt' => __('Failed to import %s: Invalid post type %s', HELPIE_DOMAIN),
                'post_already_exist' => __('%s %s already exists.', HELPIE_DOMAIN),
                'failed_to_import_post_cpt' => __('Failed to import %s: Invalid post type %s', HELPIE_DOMAIN),
                'remote_server_return_error' => __('Remote server returned error response %1$d %2$s', HELPIE_DOMAIN),
                'remote_file_too_large' => __('Remote file is too large, limit is %s', HELPIE_DOMAIN),
            ];

            return array_merge($demo_import, $onboarding);
        }

        protected static function get_backup_restore_strings()
        {
            $backupandrestore = __('Backup / Restore', HELPIE_DOMAIN);
            return [
                'backup_restore' => $backupandrestore,
                'backup_restore_settings' => sprintf(__('%s Settings', HELPIE_DOMAIN), $backupandrestore),
                'backup_restore_settings_sub' => __('Currently does not backup topic-level, article-level dynamic caps', HELPIE_DOMAIN),
                'backup_restore_settings_help' => __('The backup will download a JSON file. To import, open the JSON file and copy everything and paste below', HELPIE_DOMAIN),
            ];
        }

        protected static function get_performance_strings()
        {
            return [
                'performance' => __('Performance', HELPIE_DOMAIN),
            ];
        }

        protected static function get_insights_strings()
        {
            return [
                'name' => __('Name', HELPIE_DOMAIN),
                'words' => __('words', HELPIE_DOMAIN),
                'count' => __('count', HELPIE_DOMAIN),
                'keyword' => __('Keyword', HELPIE_DOMAIN),
                'insights' => __('Insights', HELPIE_DOMAIN),
                'searches' => __('Searches', HELPIE_DOMAIN),
                'key_users' => __("Key Users", HELPIE_DOMAIN),
                'most_happy' => __('Most Happy', HELPIE_DOMAIN),
                'most_unhappy' => __('Most Unhappy', HELPIE_DOMAIN),
                'key_articles' => __("Key Articles", HELPIE_DOMAIN),
                'lowest_rating' => __('Lowest Rating', HELPIE_DOMAIN),
                'highest_rating' => __('Highest Rating', HELPIE_DOMAIN),
                'helpie_insights' => __('Helpie Insights', HELPIE_DOMAIN),
                'happiness_score' => __('Happiness Score', HELPIE_DOMAIN),
                'most_viewed_pages' => __('Most Viewed Pages', HELPIE_DOMAIN),
                'user_happiness_index' => __('User Happiness Index', HELPIE_DOMAIN),
                'most_searched_keywords' => __('Most Searched Keywords', HELPIE_DOMAIN),
                'helpie_insights_sub' => __('Insights to a better Knowledge base.', HELPIE_DOMAIN),
                'frequently_repeated_keyword' => __('Frequently Repeated Keywords', HELPIE_DOMAIN),
            ];
        }

        protected static function get_cpt_strings()
        {
            // Single Cpt Name default to "Article"
            $settings = new \Helpie\Includes\Settings\Getters\Settings();
            $article = $settings->single_page->get_single_cpt_name();
            $articles = $settings->single_page->get_single_cpt_name_plural();

            return [
                // Updated Tags
                'updated_tag' => __('Updated Tag', HELPIE_DOMAIN),
                'updated_tags' => __('Updated Tags', HELPIE_DOMAIN),
                'search_updated_tags' => sprintf(__('Search %s', HELPIE_DOMAIN), __('Updated Tags', HELPIE_DOMAIN)),
                'all_updated_tags' => sprintf(__('All %s', HELPIE_DOMAIN), __('Updated Tags', HELPIE_DOMAIN)),
                'parent_updated_tag' => sprintf(__('Parent %s', HELPIE_DOMAIN), __('Updated Tag', HELPIE_DOMAIN)),
                'parent_updated_tag_colon' => sprintf(__('Parent %s', HELPIE_DOMAIN), __('Updated Tag', HELPIE_DOMAIN)) . ':',
                'edit_updated_tag' => sprintf(__('Edit %s', HELPIE_DOMAIN), __('Updated Tag', HELPIE_DOMAIN)),
                'update_pdated_tag' => sprintf(__('Update %s', HELPIE_DOMAIN), __('Updated Tag', HELPIE_DOMAIN)),
                'add_new_updated_tag' => sprintf(__('Add New %s', HELPIE_DOMAIN), __('Updated Tag', HELPIE_DOMAIN)),
                'new_updated_tag_name' => sprintf(__('New %s Name', HELPIE_DOMAIN), __('Updated Tag', HELPIE_DOMAIN)),
                // Added Tags
                'added_tag' => __('Added Tag', HELPIE_DOMAIN),
                'added_tags' => __('Added Tags', HELPIE_DOMAIN),
                'search_added_tags' => sprintf(__('Search %s', HELPIE_DOMAIN), __('Added Tags', HELPIE_DOMAIN)),
                'all_added_tags' => sprintf(__('All %s', HELPIE_DOMAIN), __('Added Tags', HELPIE_DOMAIN)),
                'parent_added_tag' => sprintf(__('Parent %s', HELPIE_DOMAIN), __('Added Tag', HELPIE_DOMAIN)),
                'parent_added_tag_colon' => sprintf(__('Parent %s', HELPIE_DOMAIN), __('Added Tag', HELPIE_DOMAIN)) . ':',
                'edit_added_tag' => sprintf(__('Edit %s', HELPIE_DOMAIN), __('Added Tag', HELPIE_DOMAIN)),
                'update_added_tag' => sprintf(__('Update %s', HELPIE_DOMAIN), __('Added Tag', HELPIE_DOMAIN)),
                'add_new_added_Tag' => sprintf(__('Add New %s', HELPIE_DOMAIN), __('Added Tag', HELPIE_DOMAIN)),
                'new_added_tag_name' => sprintf(__('New %s Name', HELPIE_DOMAIN), __('Added Tag', HELPIE_DOMAIN)),
                // Tags
                'tag' => __('Tag', HELPIE_DOMAIN),
                'search_tags' => sprintf(__('Search %s', HELPIE_DOMAIN), __('Tags', HELPIE_DOMAIN)),
                'all_tags' => sprintf(__('All %s', HELPIE_DOMAIN), __('Tags', HELPIE_DOMAIN)),
                'parent_tag' => sprintf(__('Parent %s', HELPIE_DOMAIN), __('Tag', HELPIE_DOMAIN)),
                'parent_tag_colon' => sprintf(__('Parent %s', HELPIE_DOMAIN), __('Tag', HELPIE_DOMAIN)) . ':',
                'edit_tag' => sprintf(__('Edit %s', HELPIE_DOMAIN), __('Tag', HELPIE_DOMAIN)),
                'update_tag' => sprintf(__('Update %s', HELPIE_DOMAIN), __('Tag', HELPIE_DOMAIN)),
                'add_new_tag' => sprintf(__('Add New %s', HELPIE_DOMAIN), __('Tag', HELPIE_DOMAIN)),
                'new_tag_name' => sprintf(__('New %s Name', HELPIE_DOMAIN), __('Tag', HELPIE_DOMAIN)),
                // Term Taxonomy -- Wiki Category
                'wiki_category' => __('Wiki Category', HELPIE_DOMAIN),
                'wiki_categories' => __('Wiki Categories', HELPIE_DOMAIN),
                'search_wiki_categories' => sprintf(__('Search %s', HELPIE_DOMAIN), __('Wiki Categories', HELPIE_DOMAIN)),
                'all_wiki_categories' => sprintf(__('All %s', HELPIE_DOMAIN), __('Wiki Categories', HELPIE_DOMAIN)),
                'parent_wiki_category' => sprintf(__('Parent %s', HELPIE_DOMAIN), __('Wiki Category', HELPIE_DOMAIN)),
                'Parent_wiki_category_colon' => sprintf(__('Parent %s', HELPIE_DOMAIN), __('Wiki Category', HELPIE_DOMAIN)) . ':',
                'edit_wiki_category' => sprintf(__('Edit %s', HELPIE_DOMAIN), __('Wiki Category', HELPIE_DOMAIN)),
                'update_wiki_category' => sprintf(__('Update %s', HELPIE_DOMAIN), __('Wiki Category', HELPIE_DOMAIN)),
                'add_new_wiki_category' => sprintf(__('Add New %s', HELPIE_DOMAIN), __('Wiki Category', HELPIE_DOMAIN)),
                'new_wiki_category_name' => sprintf(__('New %s Name', HELPIE_DOMAIN), __('Wiki Category', HELPIE_DOMAIN)),
                // Article -- Post Type
                'cpt_singular' => __($article, HELPIE_DOMAIN),
                'cpt_plural' => __($articles, HELPIE_DOMAIN),
                'helpie_kb_wiki' => __('Helpie KB Wiki', HELPIE_DOMAIN),
                'add_new_article' => sprintf(__('Add New %s', HELPIE_DOMAIN), $article),
                'edit' => __('Edit', HELPIE_DOMAIN),
                'search_articles' => sprintf(__('Search %s', HELPIE_DOMAIN), $articles),
                'no_articles_found' => sprintf(__('No %s found', HELPIE_DOMAIN), $articles),
                'parent_articles' => sprintf(__('Parent %s', HELPIE_DOMAIN), $articles),
                'filter_elements_list' => __('Filter elements list', HELPIE_DOMAIN),
                'articles_list' => sprintf(__('%s List', HELPIE_DOMAIN), $articles),
                'elements_list_navigation' => __('Elements list navigation', HELPIE_DOMAIN),
                'all_articles' => sprintf(__('All %s', HELPIE_DOMAIN), $articles),
                // cpt - addtionals
                'select_an_icon' => __('Select an Icon', HELPIE_DOMAIN),
                'add_image' => sprintf(__('%s Image', HELPIE_DOMAIN), __('Add', HELPIE_DOMAIN)),
                'remove_image' => sprintf(__('%s Image', HELPIE_DOMAIN), __('Remove', HELPIE_DOMAIN)),
                'helpie_sidebar' => __('Helpie Sidebar', HELPIE_DOMAIN),
                'helpie_sidebar_help' => sprintf(__('%s of Helpie Knowledge Base Wiki', HELPIE_DOMAIN), __('Sidebar', HELPIE_DOMAIN)),
            ];
        }

        protected static function get_envato_codecanyon_strings()
        {
            return [
                'or' => __('Or', HELPIE_DOMAIN),
                'updates' => __('Updates', HELPIE_DOMAIN),
                'days_left' => __('days left', HELPIE_DOMAIN),
                'update_now' => __('Update Now', HELPIE_DOMAIN),
                'get_support' => __('Get Support', HELPIE_DOMAIN),
                'save_changes' => __('Save Changes', HELPIE_DOMAIN),
                'extend_support' => __('Extend support', HELPIE_DOMAIN),
                'purchase_code' => __('Purchase Code', HELPIE_DOMAIN),
                'envato_api_error' => __('Envato API error:', HELPIE_DOMAIN),
                'check_for_updates' => __('Check for updates', HELPIE_DOMAIN),
                'installed_version' => __('Installed Version', HELPIE_DOMAIN),
                'plugin_activation' => __('Plugin Activation', HELPIE_DOMAIN),
                'no_update_available' => __('No update available', HELPIE_DOMAIN),
                'no_purchase_found' => __('No purchase was found', HELPIE_DOMAIN),
                'api_token_require' => __('An API token is required.', HELPIE_DOMAIN),
                'last_available_version' => __('Last Available Version', HELPIE_DOMAIN),
                'premium_ticket_support' => __('Premium Ticket Support', HELPIE_DOMAIN),
                'new_update_available' => __('A new update is available', HELPIE_DOMAIN),
                'create_personal_token' => __('Create your Personal Token', HELPIE_DOMAIN),
                'unknown_api_error' => __('An unknown API error occurred.', HELPIE_DOMAIN),
                'register_access_update' => __('Register to Access Update', HELPIE_DOMAIN),
                'unknown_error' => __('Sorry, an unknown error occurred...', HELPIE_DOMAIN),
                'enter_personal_token' => __('Please enter your Personal Token', HELPIE_DOMAIN),
                'return_to_helpie_plugin_page' => __('Return to  Helpie Plugin page.', HELPIE_DOMAIN),
                'expired_premium_ticket_support' => __('Premium Ticket Support (Expired)', HELPIE_DOMAIN),
                'enter_envato_personal_token' => __('Enter your Envato API Personal Token', HELPIE_DOMAIN),
                'error_refresh_page' => __('Sorry, an error occurred. Please refresh the page.', HELPIE_DOMAIN),
                'get_help_from_support_team' => __('Direct help from our qualified support team', HELPIE_DOMAIN),
                'unknown_api_error_from_envato' => __('An unknown API error occurred from Envato', HELPIE_DOMAIN),
                'download_new_version_from_codecanyon' => __('download new version from CodeCanyon.', HELPIE_DOMAIN),
                'enter_personal_token_to_get_updates' => __('Enter your Personal Token to get automatic Updates', HELPIE_DOMAIN),
                'envato_registeration_msg' => __('If you have a valid purchase of Helpie, you can register it to get automatic updates.', HELPIE_DOMAIN),
                'oauth_envato_protocol' => __('OAuth is a protocol that lets external apps request authorization to private details in a user\'s Envato Market account without entering their password.', HELPIE_DOMAIN),
            ];
        }

    } // END CLASS
}
